@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: selection.ml,v 1.7 2008/01/11 16:13:11 doligez Exp $ *)
+(* $Id$ *)
 
 (* Instruction selection for the ARM processor *)
 
@@ -18,6 +18,7 @@ open Misc
 open Cmm
 open Reg
 open Arch
+open Proc
 open Mach
 
 (* Immediate operands are 8-bit immediate values, zero-extended, and rotated
@@ -39,11 +40,32 @@ let is_offset n = n < 256 && n > -256
 
 let is_intconst = function Cconst_int n -> true | _ -> false
 
+(* Soft emulation of float comparisons *)
+
+let float_comparison_function = function
+  | Ceq -> "__eqdf2"
+  | Cne -> "__nedf2"
+  | Clt -> "__ltdf2"
+  | Cle -> "__ledf2"
+  | Cgt -> "__gtdf2"
+  | Cge -> "__gedf2"
+
 (* Instruction selection *)
 class selector = object(self)
 
 inherit Selectgen.selector_generic as super
 
+method! regs_for tyv =
+  (* Expand floats into pairs of integer registers *)
+  let nty = Array.length tyv in
+  let rec expand i =
+    if i >= nty then [] else begin
+      match tyv.(i) with
+      | Float -> Int :: Int :: expand (i+1)
+      | ty -> ty :: expand (i+1)
+    end in
+  Reg.createv (Array.of_list (expand 0))
+
 method is_immediate n =
   n land 0xFF = n || is_immed n 2
 
@@ -72,7 +94,7 @@ method select_shift_arith op shiftop shiftrevop args =
   | _ ->
       super#select_operation op args
 
-method select_operation op args =
+method! select_operation op args =
   match op with
     Cadda | Caddi ->
       begin match args with
@@ -114,19 +136,65 @@ method select_operation op args =
       | _ ->
         super#select_operation op args
       end
+  (* Turn floating-point operations into library function calls *)
+  | Caddf -> (Iextcall("__adddf3", false), args)
+  | Csubf -> (Iextcall("__subdf3", false), args)
+  | Cmulf -> (Iextcall("__muldf3", false), args)
+  | Cdivf -> (Iextcall("__divdf3", false), args)
+  | Cfloatofint -> (Iextcall("__floatsidf", false), args)
+  | Cintoffloat -> (Iextcall("__fixdfsi", false), args)
+  | Ccmpf comp ->
+      (Iintop_imm(Icomp(Isigned comp), 0),
+       [Cop(Cextcall(float_comparison_function comp,
+                     typ_int, false, Debuginfo.none),
+            args)])
+  (* Add coercions around loads and stores of 32-bit floats *)
+  | Cload Single ->
+      (Iextcall("__extendsfdf2", false), [Cop(Cload Word, args)])
+  | Cstore Single ->
+      begin match args with
+      | [arg1; arg2] ->
+          let arg2' =
+            Cop(Cextcall("__truncdfsf2", typ_int, false, Debuginfo.none),
+                [arg2]) in
+          self#select_operation (Cstore Word) [arg1; arg2']
+      | _ -> assert false
+      end
+  (* Other operations are regular *)
   | _ -> super#select_operation op args
 
-(* In mul rd, rm, rs,  the registers rm and rd must be different.
-   We deal with this by pretending that rm is also a result of the mul
-   operation. *)
+method! select_condition = function
+  | Cop(Ccmpf cmp, args) ->
+      (Iinttest_imm(Isigned cmp, 0),
+       Cop(Cextcall(float_comparison_function cmp,
+                    typ_int, false, Debuginfo.none),
+           args))
+  | expr ->
+      super#select_condition expr
 
-method insert_op_debug op dbg rs rd =
-  if op = Iintop(Imul) then begin
-    self#insert_debug (Iop op) dbg rs [| rd.(0); rs.(0) |]; rd
-  end else
-    super#insert_op_debug op dbg rs rd
+(* Deal with some register irregularities:
+
+1- In mul rd, rm, rs,  the registers rm and rd must be different.
+   We deal with this by pretending that rm is also a result of the mul
+   operation.
+
+2- For Inegf and Iabsf, force arguments and results in (r0, r1);
+   this simplifies code generation later.
+*)
+
+method! insert_op_debug op dbg rs rd =
+  match op with
+  | Iintop(Imul) ->
+      self#insert_debug (Iop op) dbg rs [| rd.(0); rs.(0) |]; rd
+  | Iabsf | Inegf ->
+      let r = [| phys_reg 0; phys_reg 1 |] in
+      self#insert_moves rs r;
+      self#insert_debug (Iop op) dbg r r;
+      self#insert_moves r rd;
+      rd
+  | _ ->
+      super#insert_op_debug op dbg rs rd
 
 end
 
 let fundecl f = (new selector)#emit_fundecl f
-
