@@ -10,9 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: liveness.mli,v 1.4 1999/11/17 18:56:34 xleroy Exp $ *)
+(* $Id$ *)
 
 (* Liveness analysis.
    Annotate mach code with the set of regs live at each point. *)
 
-val fundecl: Mach.fundecl -> unit
+open Format
+
+val fundecl: formatter -> Mach.fundecl -> unit
