@@ -10,16 +10,18 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printcmm.mli,v 1.5 1999/11/17 18:56:34 xleroy Exp $ *)
+(* $Id$ *)
 
 (* Pretty-printing of C-- code *)
 
-val machtype_component : Cmm.machtype_component -> unit
-val machtype : Cmm.machtype_component array -> unit
-val comparison : Cmm.comparison -> unit
-val chunk : Cmm.memory_chunk -> unit
-val operation : Cmm.operation -> unit
-val expression : Cmm.expression -> unit
-val fundecl : Cmm.fundecl -> unit
-val data : Cmm.data_item list -> unit
-val phrase : Cmm.phrase -> unit
+open Format
+
+val machtype_component : formatter -> Cmm.machtype_component -> unit
+val machtype : formatter -> Cmm.machtype_component array -> unit
+val comparison : Cmm.comparison -> string
+val chunk : Cmm.memory_chunk -> string
+val operation : Cmm.operation -> string
+val expression : formatter -> Cmm.expression -> unit
+val fundecl : formatter -> Cmm.fundecl -> unit
+val data : formatter -> Cmm.data_item list -> unit
+val phrase : formatter -> Cmm.phrase -> unit
