@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: roots.c 8828 2008-03-10 19:56:39Z xleroy $ */
+/* $Id$ */
 
 /* To walk the memory roots for garbage collection */
 
@@ -127,6 +127,7 @@ void caml_init_frame_descriptors(void)
 
 /* Communication with [caml_start_program] and [caml_call_gc]. */
 
+char * caml_top_of_stack;
 char * caml_bottom_of_stack = NULL; /* no stack initially */
 uintnat caml_last_return_address = 1; /* not in Caml code initially */
 value * caml_gc_regs;
@@ -357,3 +358,16 @@ void caml_do_local_roots(scanning_action f, char * bottom_of_stack,
     }
   }
 }
+
+uintnat (*caml_stack_usage_hook)(void) = NULL;
+
+uintnat caml_stack_usage (void)
+{
+  uintnat sz;
+  sz = (value *) caml_top_of_stack - (value *) caml_bottom_of_stack;
+  if (caml_stack_usage_hook != NULL)
+    sz += (*caml_stack_usage_hook)();
+  return sz;
+}
+
+
