@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: startup.c,v 1.33 2007/01/29 12:10:52 xleroy Exp $ */
+/* $Id$ */
 
 /* Start-up code */
 
@@ -21,46 +21,56 @@
 #include "backtrace.h"
 #include "custom.h"
 #include "fail.h"
+#include "freelist.h"
 #include "gc.h"
 #include "gc_ctrl.h"
+#include "memory.h"
 #include "misc.h"
 #include "mlvalues.h"
 #include "osdeps.h"
 #include "printexc.h"
+#include "stack.h"
 #include "sys.h"
+#include "natdynlink.h"
 #ifdef HAS_UI
 #include "ui.h"
 #endif
 
 extern int caml_parser_trace;
-header_t caml_atom_table[256];
-char * caml_static_data_start, * caml_static_data_end;
+CAMLexport header_t caml_atom_table[256];
 char * caml_code_area_start, * caml_code_area_end;
 
 /* Initialize the atom table and the static data and code area limits. */
 
 struct segment { char * begin; char * end; };
 
-static void minmax_table(struct segment *table, char **min, char **max)
-{
-  int i;
-  *min = table[0].begin;
-  *max = table[0].end;
-  for (i = 1; table[i].begin != 0; i++) {
-    if (table[i].begin < *min) *min = table[i].begin;
-    if (table[i].end > *max)   *max = table[i].end;
-  }
-}
-  
 static void init_atoms(void)
 {
-  int i;
   extern struct segment caml_data_segments[], caml_code_segments[];
+  int i;
 
-  for (i = 0; i < 256; i++) caml_atom_table[i] = Make_header(0, i, Caml_white);
-  minmax_table(caml_data_segments,
-               &caml_static_data_start, &caml_static_data_end);
-  minmax_table(caml_code_segments, &caml_code_area_start, &caml_code_area_end);
+  for (i = 0; i < 256; i++) {
+    caml_atom_table[i] = Make_header(0, i, Caml_white);
+  }
+  if (caml_page_table_add(In_static_data,
+                          caml_atom_table, caml_atom_table + 256) != 0)
+    caml_fatal_error("Fatal error: not enough memory for the initial page table");
+
+  for (i = 0; caml_data_segments[i].begin != 0; i++) {
+    if (caml_page_table_add(In_static_data,
+                            caml_data_segments[i].begin,
+                            caml_data_segments[i].end) != 0)
+      caml_fatal_error("Fatal error: not enough memory for the initial page table");
+  }
+
+  caml_code_area_start = caml_code_segments[0].begin;
+  caml_code_area_end = caml_code_segments[0].end;
+  for (i = 1; caml_code_segments[i].begin != 0; i++) {
+    if (caml_code_segments[i].begin < caml_code_area_start)
+      caml_code_area_start = caml_code_segments[i].begin;
+    if (caml_code_segments[i].end > caml_code_area_end)
+      caml_code_area_end = caml_code_segments[i].end;
+  }
 }
 
 /* Configuration parameters and flags */
@@ -98,6 +108,7 @@ static void scanmult (char *opt, uintnat *var)
 static void parse_camlrunparam(void)
 {
   char *opt = getenv ("OCAMLRUNPARAM");
+  uintnat p;
 
   if (opt == NULL) opt = getenv ("CAMLRUNPARAM");
 
@@ -111,8 +122,9 @@ static void parse_camlrunparam(void)
       case 'o': scanmult (opt, &percent_free_init); break;
       case 'O': scanmult (opt, &max_percent_free_init); break;
       case 'v': scanmult (opt, &caml_verb_gc); break;
-      case 'b': caml_init_backtrace(); break;
+      case 'b': caml_record_backtrace(Val_true); break;
       case 'p': caml_parser_trace = 1; break;
+      case 'a': scanmult (opt, &p); caml_set_allocation_policy (p); break;
       }
     }
   }
@@ -133,17 +145,20 @@ void caml_main(char **argv)
   static char proc_self_exe[256];
 #endif
   value res;
+  char tos;
 
   caml_init_ieee_floats();
   caml_init_custom_operations();
 #ifdef DEBUG
   caml_verb_gc = 63;
 #endif
+  caml_top_of_stack = &tos;
   parse_camlrunparam();
   caml_init_gc (minor_heap_init, heap_size_init, heap_chunk_init,
                 percent_free_init, max_percent_free_init);
   init_atoms();
   caml_init_signals();
+  caml_debugger_init (); /* force debugger.o stub to be linked */
   exe_name = argv[0];
   if (exe_name == NULL) exe_name = "";
 #ifdef __linux__
