@@ -10,11 +10,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printinstr.mli,v 1.4 1999/11/17 18:57:01 xleroy Exp $ *)
+(* $Id$ *)
 
 (* Pretty-print lists of instructions *)
 
 open Instruct
 
-val instruction: instruction -> unit
-val instrlist: instruction list -> unit
+open Format
+
+val instruction: formatter -> instruction -> unit
+val instrlist: formatter -> instruction list -> unit
