@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translmod.ml 8930 2008-07-24 05:35:22Z frisch $ *)
+(* $Id$ *)
 
 (* Translation from typed abstract syntax to lambda terms,
    for the module language *)
@@ -84,7 +84,7 @@ let primitive_declarations = ref ([] : Primitive.description list)
 let record_primitive = function
   | {val_kind=Val_prim p} -> primitive_declarations := p :: !primitive_declarations
   | _ -> ()
- 
+
 (* Keep track of the root path (from the root of the namespace to the
    currently compiled module expression).  Useful for naming exceptions. *)
 
@@ -109,13 +109,7 @@ let mod_prim name =
     fatal_error ("Primitive " ^ name ^ " not found.")
 
 let undefined_location loc =
-  (* Confer Translcore.assert_failed *)
-  let fname = match loc.Location.loc_start.Lexing.pos_fname with
-              | "" -> !Location.input_name
-              | x -> x in
-  let pos = loc.Location.loc_start in
-  let line = pos.Lexing.pos_lnum in
-  let char = pos.Lexing.pos_cnum - pos.Lexing.pos_bol in
+  let (fname, line, char) = Location.get_pos_info loc.Location.loc_start in
   Lconst(Const_block(0,
                      [Const_base(Const_string fname);
                       Const_base(Const_int line);
@@ -266,6 +260,8 @@ let rec transl_module cc rootpath mexp =
                 [transl_module ccarg None arg], mexp.mod_loc))
   | Tmod_constraint(arg, mty, ccarg) ->
       transl_module (compose_coercions cc ccarg) rootpath arg
+  | Tmod_unpack(arg, _) ->
+      apply_coercion cc (Translcore.transl_exp arg)
 
 and transl_structure fields cc rootpath = function
     [] ->
