@@ -11,9 +11,9 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: debugger.c 8955 2008-07-29 08:31:41Z xleroy $ */
+/* $Id$ */
 
-/* Interface with the debugger */
+/* Interface with the byte-code debugger */
 
 #ifdef _WIN32
 #include <io.h>
@@ -21,22 +21,17 @@
 
 #include <string.h>
 
+#include "alloc.h"
 #include "config.h"
 #include "debugger.h"
-#include "fail.h"
-#include "fix_code.h"
-#include "instruct.h"
-#include "intext.h"
-#include "io.h"
 #include "misc.h"
-#include "mlvalues.h"
-#include "stacks.h"
-#include "sys.h"
 
 int caml_debugger_in_use = 0;
 uintnat caml_event_count;
+int caml_debugger_fork_mode = 1; /* parent by default */
+value marshal_flags = Val_emptylist;
 
-#if !defined(HAS_SOCKETS)
+#if !defined(HAS_SOCKETS) || defined(NATIVE_CODE)
 
 void caml_debugger_init(void)
 {
@@ -46,6 +41,10 @@ void caml_debugger(enum event_kind event)
 {
 }
 
+void caml_debugger_cleanup_fork(void)
+{
+}
+
 #else
 
 #ifdef HAS_UNISTD
@@ -67,12 +66,21 @@ void caml_debugger(enum event_kind event)
 #include <process.h>
 #endif
 
+#include "fail.h"
+#include "fix_code.h"
+#include "instruct.h"
+#include "intext.h"
+#include "io.h"
+#include "mlvalues.h"
+#include "stacks.h"
+#include "sys.h"
+
 static int sock_domain;         /* Socket domain for the debugger */
 static union {                  /* Socket address for the debugger */
   struct sockaddr s_gen;
 #ifndef _WIN32
   struct sockaddr_un s_unix;
-#endif    
+#endif
   struct sockaddr_in s_inet;
 } sock_addr;
 static int sock_addr_len;       /* Length of sock_addr */
@@ -98,7 +106,7 @@ static void open_connection(void)
       setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
                  (char *) &newvalue, sizeof(newvalue));
   }
-#endif    
+#endif
   dbg_socket = socket(sock_domain, SOCK_STREAM, 0);
 #ifdef _WIN32
   if (retcode == 0) {
@@ -106,10 +114,10 @@ static void open_connection(void)
     setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
                (char *) &oldvalue, oldvaluelen);
   }
-#endif    
+#endif
   if (dbg_socket == -1 ||
       connect(dbg_socket, &sock_addr.s_gen, sock_addr_len) == -1){
-    caml_fatal_error_arg2 ("cannot connect to debugger at %s", dbg_addr,
+    caml_fatal_error_arg2 ("cannot connect to debugger at %s\n", dbg_addr,
                            "error: %s\n", strerror (errno));
   }
 #ifdef _WIN32
@@ -156,6 +164,11 @@ void caml_debugger_init(void)
   struct hostent * host;
   int n;
 
+  caml_register_global_root(&marshal_flags);
+  marshal_flags = caml_alloc(2, Tag_cons);
+  Store_field(marshal_flags, 0, Val_int(1)); /* Marshal.Closures */
+  Store_field(marshal_flags, 1, Val_emptylist);
+
   address = getenv("CAML_DEBUG_SOCKET");
   if (address == NULL) return;
   dbg_addr = address;
@@ -181,7 +194,7 @@ void caml_debugger_init(void)
         + strlen(address);
 #else
     caml_fatal_error("Unix sockets not supported");
-#endif    
+#endif
   } else {
     /* Internet domain */
     sock_domain = PF_INET;
@@ -224,7 +237,7 @@ static void safe_output_value(struct channel *chan, value val)
   saved_external_raise = caml_external_raise;
   if (sigsetjmp(raise_buf.buf, 0) == 0) {
     caml_external_raise = &raise_buf;
-    caml_output_val(chan, val, Val_unit);
+    caml_output_val(chan, val, marshal_flags);
   } else {
     /* Send wrong magic number, will cause [caml_input_value] to fail */
     caml_really_putblock(chan, "\000\000\000\000", 4);
@@ -318,7 +331,7 @@ void caml_debugger(enum event_kind event)
 #else
       caml_fatal_error("error: REQ_CHECKPOINT command");
       exit(-1);
-#endif      
+#endif
       break;
     case REQ_GO:
       caml_event_count = caml_getword(dbg_in);
@@ -332,7 +345,7 @@ void caml_debugger(enum event_kind event)
 #else
       caml_fatal_error("Fatal error: REQ_WAIT command");
       exit(-1);
-#endif      
+#endif
       break;
     case REQ_INITIAL_FRAME:
       frame = caml_extern_sp + 1;
@@ -412,8 +425,19 @@ void caml_debugger(enum event_kind event)
       caml_putword(dbg_out, (Code_val(val)-caml_start_code) * sizeof(opcode_t));
       caml_flush(dbg_out);
       break;
+    case REQ_SET_FORK_MODE:
+      caml_debugger_fork_mode = caml_getword(dbg_in);
+      break;
     }
   }
 }
 
+void caml_debugger_cleanup_fork(void)
+{
+  /* We could remove all of the breakpoints, but closing the connection
+   * means that they'll just be skipped anyway. */
+  close_connection();
+  caml_debugger_in_use = 0;
+}
+
 #endif
