@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: intern.c,v 1.61 2008/01/11 16:13:16 doligez Exp $ */
+/* $Id$ */
 
 /* Structured input, compact format */
 
@@ -19,6 +19,7 @@
 
 #include <string.h>
 #include "alloc.h"
+#include "callback.h"
 #include "custom.h"
 #include "fail.h"
 #include "gc.h"
@@ -104,7 +105,7 @@ static void intern_cleanup(void)
   if (intern_obj_table != NULL) caml_stat_free(intern_obj_table);
   if (intern_extra_block != NULL) {
     /* free newly allocated heap chunk */
-    caml_free_for_heap(intern_extra_block); 
+    caml_free_for_heap(intern_extra_block);
   } else if (intern_block != 0) {
     /* restore original header for heap block, otherwise GC is confused */
     Hd_val(intern_block) = intern_header;
@@ -121,7 +122,10 @@ static void intern_rec(value *dest)
   header_t header;
   char cksum[16];
   struct custom_operations * ops;
+  value * function_placeholder;
+  int get_function_placeholder;
 
+  get_function_placeholder = 1;
  tailcall:
   code = read8u();
   if (code >= PREFIX_SMALL_INT) {
@@ -186,7 +190,7 @@ static void intern_rec(value *dest)
       read_shared:
         Assert (ofs > 0);
         Assert (ofs <= obj_counter);
-        Assert (intern_obj_table != NULL); 
+        Assert (intern_obj_table != NULL);
         v = intern_obj_table[obj_counter - ofs];
         break;
       case CODE_SHARED16:
@@ -237,7 +241,7 @@ static void intern_rec(value *dest)
           Permute_64(v, ARCH_FLOAT_ENDIANNESS, v, 0x01234567)
         else
           Permute_64(v, ARCH_FLOAT_ENDIANNESS, v, 0x76543210);
-#endif            
+#endif
         break;
       case CODE_DOUBLE_ARRAY8_LITTLE:
       case CODE_DOUBLE_ARRAY8_BIG:
@@ -290,6 +294,15 @@ static void intern_rec(value *dest)
         ofs = read32u();
         readblock(cksum, 16);
         if (memcmp(cksum, caml_code_checksum(), 16) != 0) {
+          if (get_function_placeholder) {
+            function_placeholder =
+              caml_named_value ("Debugger.function_placeholder");
+            get_function_placeholder = 0;
+          }
+          if (function_placeholder != NULL) {
+            v = *function_placeholder;
+            break;
+          }
           intern_cleanup();
           caml_failwith("input_value: code mismatch");
         }
@@ -508,7 +521,7 @@ CAMLexport value caml_input_value_from_malloc(char * data, intnat ofs)
   intern_src = intern_input + ofs;
   intern_input_malloced = 1;
   magic = read32u();
-  if (magic != Intext_magic_number) 
+  if (magic != Intext_magic_number)
     caml_failwith("input_value_from_malloc: bad object");
   block_len = read32u();
   obj = input_val_from_block();
@@ -527,7 +540,7 @@ CAMLexport value caml_input_value_from_block(char * data, intnat len)
   intern_src = intern_input;
   intern_input_malloced = 0;
   magic = read32u();
-  if (magic != Intext_magic_number) 
+  if (magic != Intext_magic_number)
     caml_failwith("input_value_from_block: bad object");
   block_len = read32u();
   if (5*4 + block_len > len)
