@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: interp.c,v 1.97.2.1 2009/05/25 08:02:16 xleroy Exp $ */
+/* $Id$ */
 
 /* The bytecode interpreter */
 #include <stdio.h>
@@ -157,8 +157,9 @@ sp is a local copy of the global variable caml_extern_sp. */
 #define SP_REG asm("a4")
 #define ACCU_REG asm("d7")
 #endif
-#ifdef __arm__
-#define PC_REG asm("r9")
+/* PR#4953: these specific registers not available in Thumb mode */
+#if defined (__arm__) && !defined(__thumb__)
+#define PC_REG asm("r6")
 #define SP_REG asm("r8")
 #define ACCU_REG asm("r7")
 #endif
@@ -1023,14 +1024,14 @@ value caml_interprete(code_t prog, asize_t prog_size)
     Instruct(ISINT):
       accu = Val_long(accu & 1);
       Next;
-    
+
 /* Object-oriented operations */
 
 #define Lookup(obj, lab) Field (Field (obj, 0), Int_val(lab))
 
       /* please don't forget to keep below code in sync with the
-	 functions caml_cache_public_method and
-	 caml_cache_public_method2 in obj.c */
+         functions caml_cache_public_method and
+         caml_cache_public_method2 in obj.c */
 
     Instruct(GETMETHOD):
       accu = Lookup(sp[0], accu);
@@ -1136,7 +1137,7 @@ void caml_prepare_bytecode(code_t prog, asize_t prog_size) {
   Assert(prog);
   Assert(prog_size>0);
   /* actually, the threading of the bytecode might be done here */
-} 
+}
 
 void caml_release_bytecode(code_t prog, asize_t prog_size) {
   /* other implementations of the interpreter (such as an hypothetical
