@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: memory.h,v 1.59.4.1 2008/11/02 14:30:05 xleroy Exp $ */
+/* $Id$ */
 
 /* Allocation macros and functions */
 
@@ -168,8 +168,8 @@ CAMLextern struct caml__roots_block *caml_local_roots;  /* defined in roots.c */
 
    If you need local variables of type [value], declare them with one
    or more calls to the [CAMLlocal] macros at the beginning of the
-   function. Use [CAMLlocalN] (at the beginning of the function) to
-   declare an array of [value]s.
+   function, after the call to CAMLparam.  Use [CAMLlocalN] (at the
+   beginning of the function) to declare an array of [value]s.
 
    Your function may raise an exception or return a [value] with the
    [CAMLreturn] macro.  Its argument is simply the [value] returned by
