@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: printexc.c 7812 2007-01-29 12:11:18Z xleroy $ */
+/* $Id$ */
 
 /* Print an uncaught exception and abort */
 
@@ -44,7 +44,7 @@ static void add_string(struct stringbuf *buf, char *s)
   if (len > 0) memmove(buf->ptr, s, len);
   buf->ptr += len;
 }
-  
+
 CAMLexport char * caml_format_exception(value exn)
 {
   mlsize_t start, i;
@@ -60,7 +60,8 @@ CAMLexport char * caml_format_exception(value exn)
     /* Check for exceptions in the style of Match_failure and Assert_failure */
     if (Wosize_val(exn) == 2 &&
         Is_block(Field(exn, 1)) &&
-        Tag_val(Field(exn, 1)) == 0) {
+        Tag_val(Field(exn, 1)) == 0 &&
+        caml_is_special_exception(Field(exn, 0))) {
       bucket = Field(exn, 1);
       start = 0;
     } else {
