@@ -6,21 +6,22 @@
 /*                                                                     */
 /*  Copyright 2001 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: printexc.h,v 1.1 2001/06/15 14:22:38 xleroy Exp $ */
+/* $Id$ */
 
-#ifndef _printexc_
-#define _printexc_
+#ifndef CAML_PRINTEXC_H
+#define CAML_PRINTEXC_H
 
 
 #include "misc.h"
 #include "mlvalues.h"
 
-char * format_caml_exception (value);
-void fatal_uncaught_exception (value) Noreturn;
+CAMLextern char * caml_format_exception (value);
+void caml_fatal_uncaught_exception (value) Noreturn;
 
 
-#endif /* _printexc_ */
+#endif /* CAML_PRINTEXC_H */
