@@ -133,7 +133,9 @@ module Make (Ast : Sig.Camlp4Ast) = struct
 
     let rec self i acc =
       match i with
-      [ <:ident< $i1$.$i2$ >> ->
+      [ <:ident< $lid:"*predef*"$.$lid:"option"$ >> ->
+          (ldot (lident "*predef*") "option", `lident)
+      | <:ident< $i1$.$i2$ >> ->
           self i2 (Some (self i1 acc))
       | <:ident< $i1$ $i2$ >> ->
           let i' = Lapply (fst (self i1 None)) (fst (self i2 None)) in
@@ -204,6 +206,9 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     | <:ctyp< '$s$ >> -> [s]
     | _ -> assert False ];
 
+  value predef_option loc =
+    TyId (loc, IdAcc (loc, IdLid (loc, "*predef*"), IdLid (loc, "option")));
+
   value rec ctyp =
     fun
     [ TyId loc i ->
@@ -226,7 +231,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     | TyArr loc (TyLab _ lab t1) t2 ->
         mktyp loc (Ptyp_arrow lab (ctyp t1) (ctyp t2))
     | TyArr loc (TyOlb loc1 lab t1) t2 ->
-        let t1 = TyApp loc1 <:ctyp@loc1< option >> t1 in
+        let t1 = TyApp loc1 (predef_option loc1) t1 in
         mktyp loc (Ptyp_arrow ("?" ^ lab) (ctyp t1) (ctyp t2))
     | TyArr loc t1 t2 -> mktyp loc (Ptyp_arrow "" (ctyp t1) (ctyp t2))
     | <:ctyp@loc< < $fl$ > >> -> mktyp loc (Ptyp_object (meth_list fl []))
@@ -346,7 +351,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
         mktype loc tl cl Ptype_abstract (mkprivate' pflag) m ]
   ;
 
-  value type_decl tl cl t = type_decl tl cl (loc_of_ctyp t) None False t;
+  value type_decl tl cl t loc = type_decl tl cl loc None False t;
 
   value mkvalue_desc t p = {pval_type = ctyp t; pval_prim = p};
 
@@ -536,7 +541,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     | <:patt@loc< ($tup:_$) >> -> error loc "singleton tuple pattern"
     | PaTyc loc p t -> mkpat loc (Ppat_constraint (patt p) (ctyp t))
     | PaTyp loc i -> mkpat loc (Ppat_type (long_type_ident i))
-    | PaVrn loc s -> mkpat loc (Ppat_variant s None)
+    | PaVrn loc s -> mkpat loc (Ppat_variant (conv_con s) None)
     | PaLaz loc p -> mkpat loc (Ppat_lazy (patt p))
     | PaEq _ _ _ | PaSem _ _ _ | PaCom _ _ _ | PaNil _ as p ->
         error (loc_of_patt p) "invalid pattern" ]
@@ -769,7 +774,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     | <:expr@loc< $uid:s$ >> ->
         (* let ca = constructors_arity () in *)
         mkexp loc (Pexp_construct (lident (conv_con s)) None True)
-    | ExVrn loc s -> mkexp loc (Pexp_variant s None)
+    | ExVrn loc s -> mkexp loc (Pexp_variant (conv_con s) None)
     | ExWhi loc e1 el ->
         let e2 = ExSeq loc el in
         mkexp loc (Pexp_while (expr e1) (expr e2))
@@ -835,7 +840,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     match x with
     [ <:ctyp< $x$ and $y$ >> ->
          mktype_decl x (mktype_decl y acc)
-    | Ast.TyDcl _ c tl td cl ->
+    | Ast.TyDcl loc c tl td cl ->
         let cl =
           List.map
             (fun (t1, t2) ->
@@ -843,7 +848,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
               (ctyp t1, ctyp t2, mkloc loc))
             cl
         in
-        [(c, type_decl (List.fold_right type_parameters tl []) cl td) :: acc]
+        [(c, type_decl (List.fold_right type_parameters tl []) cl td loc) :: acc]
     | _ -> assert False ]
   and module_type =
     fun
@@ -942,6 +947,8 @@ module Make (Ast : Sig.Camlp4Ast) = struct
                       (List.map ctyp (list_of_ctyp t []))) :: l ]
     | <:str_item@loc< exception $uid:s$ = $i$ >> ->
         [mkstr loc (Pstr_exn_rebind (conv_con s) (ident i)) :: l ]
+    | <:str_item@loc< exception $uid:_$ of $_$ = $_$ >> ->
+        error loc "type in exception alias"
     | StExc _ _ _ -> assert False (*FIXME*)
     | StExp loc e -> [mkstr loc (Pstr_eval (expr e)) :: l]
     | StExt loc n t sl -> [mkstr loc (Pstr_primitive n (mkvalue_desc t (list_of_meta_list sl))) :: l]
@@ -964,7 +971,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     | CtFun loc (TyLab _ lab t) ct ->
         mkcty loc (Pcty_fun lab (ctyp t) (class_type ct))
     | CtFun loc (TyOlb loc1 lab t) ct ->
-        let t = TyApp loc1 <:ctyp@loc1< option >> t in
+        let t = TyApp loc1 (predef_option loc1) t in
         mkcty loc (Pcty_fun ("?" ^ lab) (ctyp t) (class_type ct))
     | CtFun loc t ct -> mkcty loc (Pcty_fun "" (ctyp t) (class_type ct))
     | CtSig loc t_o ctfl ->
