@@ -21,7 +21,7 @@ open Camlp4;                                        (* -*- camlp4r -*- *)
 
 module Id : Sig.Id = struct
   value name = "Camlp4OCamlParser";
-  value version = "$Id: Camlp4OCamlParser.ml,v 1.3.2.19 2007/12/18 08:53:26 ertai Exp $";
+  value version = Sys.ocaml_version;
 end;
 
 module Make (Syntax : Sig.Camlp4Syntax) = struct
@@ -59,20 +59,6 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
                     (Ast.loc_of_expr e2) in
         <:expr< do { $e1$; $e2$ } >> ];
 
-  value test_constr_decl =
-    Gram.Entry.of_parser "test_constr_decl"
-      (fun strm ->
-        match Stream.npeek 1 strm with
-        [ [(UIDENT _, _)] ->
-            match Stream.npeek 2 strm with
-            [ [_; (KEYWORD ".", _)] -> raise Stream.Failure
-            | [_; (KEYWORD "(", _)] -> raise Stream.Failure
-            | [_ :: _] -> ()
-            | _ -> raise Stream.Failure ]
-        | [(KEYWORD "|", _)] -> ()
-        | _ -> raise Stream.Failure ])
-  ;
-
   value stream_peek_nth n strm =
     loop n (Stream.npeek n strm) where rec loop n =
       fun
@@ -81,6 +67,12 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       | [_ :: l] -> loop (n - 1) l ]
   ;
 
+  value test_not_dot_nor_lparen =
+    Gram.Entry.of_parser "test_not_dot_nor_lparen" (fun strm ->
+      match Stream.peek strm with
+      [ Some (KEYWORD ("."|"("),_) -> raise Stream.Failure
+      | _ -> () ]);
+
   (* horrible hacks to be able to parse class_types *)
 
   value test_ctyp_minusgreater =
@@ -115,39 +107,14 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | _ -> 1 ])
   ;
 
-  value test_label_eq =
-    Gram.Entry.of_parser "test_label_eq"
-      (test 1 where rec test lev strm =
-        match stream_peek_nth lev strm with
-        [ Some (UIDENT _ | LIDENT _ | KEYWORD ".") ->
-            test (lev + 1) strm
-        | Some (KEYWORD "=") -> ()
-        | _ -> raise Stream.Failure ])
-  ;
-
-  value test_typevar_list_dot =
-    Gram.Entry.of_parser "test_typevar_list_dot"
-      (let rec test lev strm =
-        match stream_peek_nth lev strm with
-        [ Some (KEYWORD "'") -> test2 (lev + 1) strm
-        | Some (KEYWORD ".") -> ()
-        | _ -> raise Stream.Failure ]
-      and test2 lev strm =
-        match stream_peek_nth lev strm with
-        [ Some (UIDENT _ | LIDENT _) -> test (lev + 1) strm
-        | _ -> raise Stream.Failure ]
-      in
-      test 1)
-  ;
-
-  value lident_colon =
-    Gram.Entry.of_parser "lident_colon"
-      (fun strm ->
-        match Stream.npeek 2 strm with
-        [ [(LIDENT i, _); (KEYWORD ":", _)] ->
-            do { Stream.junk strm; Stream.junk strm; i }
-        | _ -> raise Stream.Failure ])
-  ;
+  value lident_colon =	 
+     Gram.Entry.of_parser "lident_colon"	 
+       (fun strm ->	 
+         match Stream.npeek 2 strm with	 
+         [ [(LIDENT i, _); (KEYWORD ":", _)] ->	 
+             do { Stream.junk strm; Stream.junk strm; i }	 
+         | _ -> raise Stream.Failure ])	 
+   ;
 
   value rec is_ident_constr_call =
     fun
@@ -175,16 +142,18 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
   DELETE_RULE Gram module_type: SELF; SELF; dummy END;
   DELETE_RULE Gram module_type: SELF; "."; SELF END;
   DELETE_RULE Gram label_expr: label_longident; fun_binding END;
+  DELETE_RULE Gram meth_list: meth_decl; opt_dot_dot END;
   DELETE_RULE Gram expr: "let"; opt_rec; binding; "in"; SELF END;
   DELETE_RULE Gram expr: "let"; "module"; a_UIDENT; module_binding0; "in"; SELF END;
+  DELETE_RULE Gram expr: "let"; "open"; module_longident; "in"; SELF END;
   DELETE_RULE Gram expr: "fun"; "["; LIST0 match_case0 SEP "|"; "]" END;
   DELETE_RULE Gram expr: "if"; SELF; "then"; SELF; "else"; SELF END;
   DELETE_RULE Gram expr: "do"; do_sequence END;
   DELETE_RULE Gram expr: SELF; SELF END;
   DELETE_RULE Gram expr: "new"; class_longident END;
   DELETE_RULE Gram expr: "["; sem_expr_for_list; "::"; expr; "]" END;
-  DELETE_RULE Gram expr: "{"; label_expr; "}" END;
-  DELETE_RULE Gram expr: "{"; "("; SELF; ")"; "with"; label_expr; "}" END;
+  DELETE_RULE Gram expr: "{"; label_expr_list; "}" END;
+  DELETE_RULE Gram expr: "{"; "("; SELF; ")"; "with"; label_expr_list; "}" END;
   DELETE_RULE Gram expr: "("; SELF; ","; comma_expr; ")" END;
   DELETE_RULE Gram expr: SELF; ":="; SELF; dummy END;
   DELETE_RULE Gram expr: "~"; a_LIDENT; ":"; SELF END;
@@ -217,6 +186,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
   clear star_ctyp;
   clear match_case;
   clear with_constr;
+  clear package_type;
   clear top_phrase;
 
   EXTEND Gram
@@ -234,10 +204,11 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       comma_ctyp comma_expr comma_ipatt comma_patt comma_type_parameter
       constrain constructor_arg_list constructor_declaration
       constructor_declarations ctyp ctyp_quot cvalue_binding direction_flag
-      dummy eq_expr expr expr_eoi expr_quot field_expr fun_binding
+      dummy eq_expr expr expr_eoi expr_quot fun_binding
       fun_def ident ident_quot implem interf ipatt ipatt_tcon label
-      label_declaration label_expr label_ipatt label_longident label_patt
-      labeled_ipatt let_binding meth_list module_binding module_binding0
+      label_declaration label_declaration_list label_expr label_expr_list
+      label_longident label_patt_list meth_list
+      labeled_ipatt let_binding module_binding module_binding0
       module_binding_quot module_declaration module_expr module_expr_quot
       module_longident module_longident_with_app module_rec_declaration
       module_type module_type_quot more_ctyp name_tags opt_as_lident
@@ -252,7 +223,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       type_ident_and_parameters type_kind type_longident
       type_longident_and_parameters type_parameter type_parameters typevars
       use_file val_longident value_let value_val with_constr with_constr_quot
-      infixop0 infixop1 infixop2 infixop3 infixop4 do_sequence
+      infixop0 infixop1 infixop2 infixop3 infixop4 do_sequence package_type
     ;
     sem_expr:
       [ [ e1 = expr LEVEL "top"; ";"; e2 = SELF -> <:expr< $e1$; $e2$ >>
@@ -282,13 +253,17 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
               | _ -> <:str_item< value $rec:r$ $bi$ >> ]
           | "let"; "module"; m = a_UIDENT; mb = module_binding0; "in"; e = expr ->
               <:str_item< let module $m$ = $mb$ in $e$ >>
+          | "let"; "open"; i = module_longident; "in"; e = expr ->
+              <:str_item< let open $id:i$ in $e$ >>
       ] ]
     ;
-    expr: BEFORE "top"
-      [ ";" RIGHTA
-        [ e1 = SELF; ";"; e2 = SELF ->
+    seq_expr:
+      [ [ e1 = expr LEVEL "top"; ";"; e2 = SELF ->
             conc_seq e1 e2
-        | e1 = SELF; ";" -> e1 ] ];
+        | e1 = expr LEVEL "top"; ";" -> e1
+        | e1 = expr LEVEL "top" -> e1 ] ];
+    expr: BEFORE "top"
+      [ ";" [ e = seq_expr -> e ] ];
     expr: LEVEL "top"
       [ [ "let"; r = opt_rec; bi = binding; "in";
           x = expr LEVEL ";" ->
@@ -296,6 +271,8 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | "let"; "module"; m = a_UIDENT; mb = module_binding0; "in";
           e = expr LEVEL ";" ->
             <:expr< let module $m$ = $mb$ in $e$ >>
+        | "let"; "open"; i = module_longident; "in"; e = expr LEVEL ";" ->
+            <:expr< let open $id:i$ in $e$ >>
         | "function"; a = match_case ->
             <:expr< fun [ $a$ ] >>
         | "if"; e1 = SELF; "then"; e2 = expr LEVEL "top";
@@ -306,8 +283,8 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       ] ];
     expr: BEFORE "||"
       [ ","
-        [ e = SELF; ","; el = (*FIXME comma_expr*)LIST1 NEXT SEP "," ->
-            <:expr< ( $e$, $Ast.exCom_of_list el$ ) >> ]
+        [ e1 = SELF; ","; e2 = comma_expr ->
+            <:expr< ( $e1$, $e2$ ) >> ]
       | ":=" NONA
         [ e1 = SELF; ":="; e2 = expr LEVEL "top" ->
             <:expr< $e1$.val := $e2$ >>
@@ -331,9 +308,9 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
     expr: LEVEL "simple" (* LEFTA *)
       [ [ "false" -> <:expr< False >>
         | "true" -> <:expr< True >>
-        | "{"; test_label_eq; lel = label_expr; "}" ->
+        | "{"; lel = TRY [lel = label_expr_list; "}" -> lel] ->
             <:expr< { $lel$ } >>
-        | "{"; e = expr LEVEL "."; "with"; lel = label_expr; "}" ->
+        | "{"; e = TRY [e = expr LEVEL "."; "with" -> e]; lel = label_expr_list; "}" ->
             <:expr< { ($e$) with $lel$ } >>
         | "new"; i = class_longident -> <:expr< new $i$ >>
       ] ]
@@ -372,6 +349,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
                 List.fold_left (fun p1 p2 -> <:patt< $p1$ $p2$ >>) p1
                                 (Ast.list_of_patt p [])
             | _ -> <:patt< $p1$ $p2$ >> ]
+        | "lazy"; p = SELF -> <:patt< lazy $p$ >>
         | `ANTIQUOT (""|"pat"|"anti" as n) s ->
             <:patt< $anti:mk_anti ~c:"patt" n s$ >>
         | p = patt_constr -> p ]
@@ -404,7 +382,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
             mk_list <:patt< [] >>
         | "[|"; "|]" -> <:patt< [||] >>
         | "[|"; pl = sem_patt; "|]" -> <:patt< [| $pl$ |] >>
-        | "{"; pl = label_patt; "}" -> <:patt< { $pl$ } >>
+        | "{"; pl = label_patt_list; "}" -> <:patt< { $pl$ } >>
         | "("; ")" -> <:patt< () >>
         | "("; p = patt; ":"; t = ctyp; ")" -> <:patt< ($p$ : $t$) >>
         | "("; p = patt; ")" -> <:patt< $p$ >>
@@ -412,10 +390,10 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | "`"; s = a_ident -> <:patt< ` $s$ >>
         | "#"; i = type_longident -> <:patt< # $i$ >> ] ]
     ;
-    (* comma_expr:
-      [ [ e1 = SELF; ","; e2 = SELF -> <:expr< $e1$, $e2$ >>
-        | e = expr LEVEL ":=" -> e ] ]
-    ;                                                           *)
+    comma_expr:
+      [ [ e1 = expr LEVEL ":="; ","; e2 = SELF -> <:expr< $e1$, $e2$ >>
+        | e1 = expr LEVEL ":=" -> e1 ] ]
+    ;
     (* comma_patt:
       [ [ p1 = SELF; ","; p2 = SELF -> <:patt< $p1$, $p2$ >>
         | p = patt LEVEL ".." -> p ] ]
@@ -434,7 +412,30 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         | "type"; t1 = type_longident_and_parameters; "="; t2 = opt_private_ctyp ->
             <:with_constr< type $t1$ = $t2$ >>
         | "module"; i1 = module_longident; "="; i2 = module_longident_with_app ->
-            <:with_constr< module $i1$ = $i2$ >> ] ]
+            <:with_constr< module $i1$ = $i2$ >>
+        | "type"; `ANTIQUOT (""|"typ"|"anti" as n) s; ":="; t = opt_private_ctyp ->
+            <:with_constr< type $anti:mk_anti ~c:"ctyp" n s$ := $t$ >>
+        | "type"; t1 = type_longident_and_parameters; ":="; t2 = opt_private_ctyp ->
+            <:with_constr< type $t1$ := $t2$ >>
+        | "module"; i1 = module_longident; ":="; i2 = module_longident_with_app ->
+            <:with_constr< module $i1$ := $i2$ >> ] ]
+    ;
+    package_type:
+      [ [ i = module_longident_with_app -> <:module_type< $id:i$ >>
+        | i = module_longident_with_app; "with"; cs = package_type_cstrs ->
+            <:module_type< $id:i$ with $cs$ >>
+      ] ]
+    ;
+    package_type_cstr:
+      [ [ "type"; i = a_LIDENT; "="; ty = ctyp ->
+            <:with_constr< type $lid:i$ = $ty$ >>
+      ] ]
+    ;
+    package_type_cstrs:
+      [ [ c = package_type_cstr -> c
+        | c = package_type_cstr; "and"; cs = package_type_cstrs ->
+            <:with_constr< $c$ and $cs$ >>
+      ] ]
     ;
     opt_private_ctyp:
       [ [ "private"; t = ctyp -> <:ctyp< private $t$ >>
@@ -444,7 +445,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       [ [ i = lident_colon; t = ctyp LEVEL "star"; "->"; ct = SELF ->
             <:class_type< [ ~ $i$ : $t$ ] -> $ct$ >>
         | "?"; i = a_LIDENT; ":"; t = ctyp LEVEL "star"; "->"; ct = SELF ->
-            <:class_type< [ ? $i$ : $t$ ] -> $ct$ >> 
+            <:class_type< [ ? $i$ : $t$ ] -> $ct$ >>
         | i = OPTLABEL (* FIXME inline a_OPTLABEL *); t = ctyp LEVEL "star"; "->"; ct = SELF ->
             <:class_type< [ ? $i$ : $t$ ] -> $ct$ >>
         | test_ctyp_minusgreater; t = ctyp LEVEL "star"; "->"; ct = SELF ->
@@ -471,7 +472,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       [ [ t1 = SELF; "as"; "'"; i = a_ident -> <:ctyp< $t1$ as '$i$ >> ]
       | "arrow" RIGHTA
         [ t1 = SELF; "->"; t2 = SELF -> <:ctyp< $t1$ -> $t2$ >>
-        | i = lident_colon; t1 = ctyp LEVEL "star"; "->"; t2 = SELF ->
+        | i = TRY [i = a_LIDENT; ":" -> i]; t1 = ctyp LEVEL "star"; "->"; t2 = SELF ->
             <:ctyp< ( ~ $i$ : $t1$ ) -> $t2$ >>
         | i = a_OPTLABEL; t1 = ctyp LEVEL "star"; "->"; t2 = SELF ->
             <:ctyp< ( ? $i$ : $t1$ ) -> $t2$ >>
@@ -507,8 +508,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
             mk <:ctyp< $i$ $t$ >>
         | "("; t = SELF; ")" -> <:ctyp< $t$ >>
         | "#"; i = class_longident -> <:ctyp< # $i$ >>
-        | "<"; ml = opt_meth_list; v = opt_dot_dot; ">" ->
-            <:ctyp< < $ml$ $..:v$ > >>
+        | "<"; t = opt_meth_list; ">" -> t
         | "["; OPT "|"; rfl = row_field; "]" ->
             <:ctyp< [ = $rfl$ ] >>
         | "["; ">"; "]" -> <:ctyp< [ > $<:ctyp<>>$ ] >>
@@ -518,8 +518,11 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
             <:ctyp< [ < $rfl$ ] >>
         | "[<"; OPT "|"; rfl = row_field; ">"; ntl = name_tags; "]" ->
             <:ctyp< [ < $rfl$ > $ntl$ ] >>
+        | "("; "module"; p = package_type; ")" -> <:ctyp< (module $p$) >>
         ] ]
     ;
+    meth_list:
+      [ [ m = meth_decl -> (m, <:row_var_flag<>>) ] ];
     comma_ctyp_app:
       [ [ t1 = ctyp; ","; t2 = SELF -> fun acc -> t2 <:ctyp< $acc$ $t1$ >>
         | t = ctyp -> fun acc -> <:ctyp< $acc$ $t$ >>
@@ -564,18 +567,30 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
     ;
     type_kind:
       [ [ "private"; tk = type_kind -> <:ctyp< private $tk$ >>
-        | test_constr_decl; OPT "|";
-          t = constructor_declarations -> <:ctyp< [ $t$ ] >>
-        | t = ctyp -> <:ctyp< $t$ >>
-        | t = ctyp; "="; "private"; tk = type_kind ->
+        | (x, t) = TRY [x = OPT "|"; t = constructor_declarations;
+                        test_not_dot_nor_lparen -> (x, t)] ->
+            (* If there is no "|" and [t] is an antiquotation,
+               then it is not a sum type. *)
+            match (x, t) with
+            [ (None, Ast.TyAnt _) -> t
+            | _ -> <:ctyp< [ $t$ ] >> ]
+        | t = TRY ctyp -> <:ctyp< $t$ >>
+        | t = TRY ctyp; "="; "private"; tk = type_kind ->
             <:ctyp< $t$ == private $tk$ >>
-        | t1 = ctyp; "="; "{"; t2 = label_declaration; "}" ->
+        | t1 = TRY ctyp; "="; "{"; t2 = label_declaration_list; "}" ->
             <:ctyp< $t1$ == { $t2$ } >>
-        | t1 = ctyp; "="; OPT "|"; t2 = constructor_declarations ->
+        | t1 = TRY ctyp; "="; OPT "|"; t2 = constructor_declarations ->
             <:ctyp< $t1$ == [ $t2$ ] >>
-        | "{"; t = label_declaration; "}" ->
+        | "{"; t = label_declaration_list; "}" ->
             <:ctyp< { $t$ } >> ] ]
     ;
+    ctyp_quot:
+      [ [ "private"; t = ctyp_quot -> <:ctyp< private $t$ >>
+        | "|"; t = constructor_declarations -> <:ctyp< [ $t$ ] >>
+        | x = more_ctyp; "="; y = ctyp_quot -> <:ctyp< $x$ == $y$ >>
+        | "{"; t = label_declaration_list; "}" -> <:ctyp< { $t$ } >>
+      ] ]
+    ;
     module_expr: LEVEL "apply"
       [ [ i = SELF; "("; j = SELF; ")" -> <:module_expr< $i$ $j$ >> ] ]
     ;
@@ -600,9 +615,7 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       [ [ "val" -> () ] ]
     ;
     label_declaration:
-      [ LEFTA
-        [ t1 = SELF; ";"; t2 = SELF -> <:ctyp< $t1$; $t2$ >>
-        | `ANTIQUOT (""|"typ" as n) s ->
+      [ [ `ANTIQUOT (""|"typ" as n) s ->
             <:ctyp< $anti:mk_anti ~c:"ctyp" n s$ >>
         | `QUOTATION x -> Quotation.expand _loc x Quotation.DynAst.ctyp_tag
         | s = a_LIDENT; ":"; t = poly_type ->  <:ctyp< $lid:s$ : $t$ >>
@@ -611,9 +624,9 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
       ] ]
     ;
     poly_type:
-      [ [ test_typevar_list_dot; t1 = typevars; "."; t2 = ctyp ->
+      [ [ t1 = TRY [t = typevars; "." -> t]; t2 = ctyp ->
             <:ctyp< ! $t1$ . $t2$ >>
-        | t = ctyp -> t ] ]
+        | t = TRY ctyp -> t ] ]
     ;
     labeled_ipatt:
       [ [ i = a_LABEL; p = patt LEVEL "simple" ->
@@ -625,6 +638,8 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
             <:patt< ~ $i$ : ($lid:i$ : $t$) >>
         | i = a_OPTLABEL; j = a_LIDENT -> (* ?a:b <> ?a : b *)
             <:patt< ? $i$ : ($lid:j$) >>
+        | i = a_OPTLABEL; "_" ->
+            <:patt< ? $i$ : (_) >>
         | i = a_OPTLABEL; "("; p = patt; ")" ->
             <:patt< ? $i$ : ($p$) >>
         | i = a_OPTLABEL; "("; p = patt; "="; e = expr; ")" ->
