@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: debugger_config.ml 9547 2010-01-22 12:48:24Z doligez $ *)
+(* $Id$ *)
 
 (**************************** Configuration file ***********************)
 
@@ -80,3 +80,7 @@ let make_checkpoints = ref
   (match Sys.os_type with
     "Win32" -> false
   | _ -> true)
+
+(*** Environment variables for debugee. ***)
+
+let environment = ref []
