@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: debugger_config.mli 9221 2009-04-02 09:06:33Z xclerc $ *)
+(* $Id$ *)
 
 (********************** Configuration file *****************************)
 
@@ -34,3 +34,6 @@ val checkpoint_small_step : int64 ref
 val checkpoint_max_count : int ref
 val make_checkpoints : bool ref
 
+(*** Environment variables for debugee. ***)
+
+val environment : string list ref
