@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: frames.ml,v 1.10.18.1 2009/04/02 09:44:21 xclerc Exp $ *)
+(* $Id$ *)
 
 (***************************** Frames **********************************)
 
@@ -33,7 +33,9 @@ let selected_point () =
     None ->
       raise Not_found
   | Some ev ->
-      (ev.ev_module, (Events.get_pos ev).Lexing.pos_cnum)
+      (ev.ev_module,
+       (Events.get_pos ev).Lexing.pos_lnum,
+       (Events.get_pos ev).Lexing.pos_cnum - (Events.get_pos ev).Lexing.pos_bol)
 
 let selected_event_is_before () =
   match !selected_event with
@@ -73,7 +75,7 @@ let select_frame frame_number =
         | _ ->
             set_initial_frame();
             selected_event := Some(move_up frame_number curr_event);
-            current_frame := frame_number          
+            current_frame := frame_number
   with Not_found ->
     set_frame initial_sp;
     raise Not_found
