@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main.ml 9310 2009-07-08 14:30:05Z xclerc $ *)
+(* $Id$ *)
 
 open Input_handling
 open Question
@@ -161,6 +161,10 @@ let print_version () =
   printf "The Objective Caml debugger, version %s@." Sys.ocaml_version;
   exit 0;
 ;;
+let print_version_num () =
+  printf "%s@." Sys.ocaml_version;
+  exit 0;
+;;
 
 let speclist = [
    "-c", Arg.Int set_checkpoints,
@@ -175,13 +179,19 @@ let speclist = [
       "<filename>  Set the name of the communication socket";
    "-version", Arg.Unit print_version,
       " Print version and exit";
+   "-vnum", Arg.Unit print_version_num,
+      " Print version number and exit";
    ]
 
+let function_placeholder () =
+  raise Not_found
+
 let main () =
+  Callback.register "Debugger.function_placeholder" function_placeholder;
   try
-    socket_name := 
+    socket_name :=
       (match Sys.os_type with
-        "Win32" -> 
+        "Win32" ->
           (Unix.string_of_inet_addr Unix.inet_addr_loopback)^
           ":"^
           (string_of_int (10000 + ((Unix.getpid ()) mod 10000)))
