@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: source.ml 9540 2010-01-20 16:26:46Z doligez $ *)
+(* $Id$ *)
 
 (************************ Source management ****************************)
 
@@ -39,20 +39,7 @@ let source_of_module pos mdle =
       Debugger_config.load_path_for
       !Config.load_path in
   let fname = pos.Lexing.pos_fname in
-  if fname = "" then
-    let innermost_module =
-      try
-        let dot_index = String.rindex mdle '.' in
-        String.sub mdle (succ dot_index) (pred ((String.length mdle) - dot_index))
-      with Not_found -> mdle in
-    let rec loop =
-      function
-      | [] -> raise Not_found
-      | ext :: exts ->
-          try find_in_path_uncap path (innermost_module ^ ext)
-          with Not_found -> loop exts
-    in loop source_extensions
-  else if Filename.is_implicit fname then
+  if Filename.is_implicit fname then
     find_in_path path fname
   else
     fname
