@@ -11,13 +11,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: source.mli,v 1.2 1999/11/17 18:57:28 xleroy Exp $ *)
+(* $Id$ *)
 
 (************************ Source management ****************************)
 
 (*** Conversion function. ***)
 
-val source_of_module: string -> string
+val source_of_module: Lexing.position -> string -> string
 
 (*** buffer cache ***)
 
@@ -27,7 +27,7 @@ val buffer_max_count : int ref
 
 val flush_buffer_list : unit -> unit
 
-val get_buffer : string -> buffer
+val get_buffer : Lexing.position -> string -> buffer
 
 val buffer_content : buffer -> string
 val buffer_length : buffer -> int
@@ -56,3 +56,6 @@ val pos_of_line : buffer -> int -> position
 (* Convert a coordinate (line / column) into a position. *)
 (* --- The first line and column are line 1 and column 1. *)
 val point_of_coord : buffer -> int -> int -> int
+
+(* Return the offsets of both line start and cnum for the passed position. *)
+val start_and_cnum : buffer -> Lexing.position -> (int * int)
