@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: optcompile.ml,v 1.53 2005/08/08 09:41:51 xleroy Exp $ *)
+(* $Id$ *)
 
 (* The batch compiler *)
 
@@ -43,14 +43,37 @@ let initial_env () =
     then Env.initial
     else Env.open_pers_signature "Pervasives" Env.initial
   with Not_found ->
-    fatal_error "cannot open Pervasives.cmi"
+    fatal_error "cannot open pervasives.cmi"
+
+(* Note: this function is duplicated in compile.ml *)
+let check_unit_name ppf filename name =
+  try
+    begin match name.[0] with
+    | 'A'..'Z' -> ()
+    | _ ->
+       Location.print_warning (Location.in_file filename) ppf
+        (Warnings.Bad_module_name name);
+       raise Exit;
+    end;
+    for i = 1 to String.length name - 1 do
+      match name.[i] with
+      | 'A'..'Z' | 'a'..'z' | '0'..'9' | '_' | '\'' -> ()
+      | _ ->
+         Location.print_warning (Location.in_file filename) ppf
+           (Warnings.Bad_module_name name);
+         raise Exit;
+    done;
+  with Exit -> ()
+;;
 
 (* Compile a .mli file *)
 
 let interface ppf sourcefile outputprefix =
+  Location.input_name := sourcefile;
   init_path ();
   let modulename =
     String.capitalize(Filename.basename(chop_extension_if_any sourcefile)) in
+  check_unit_name ppf sourcefile modulename;
   Env.set_unit_name modulename;
   let inputfile = Pparse.preprocess sourcefile in
   try
@@ -64,9 +87,11 @@ let interface ppf sourcefile outputprefix =
     Warnings.check_fatal ();
     if not !Clflags.print_types then
       Env.save_signature sg modulename (outputprefix ^ ".cmi");
-    Pparse.remove_preprocessed inputfile
+    Pparse.remove_preprocessed inputfile;
+    Stypes.dump (outputprefix ^ ".annot");
   with e ->
     Pparse.remove_preprocessed_if_ast inputfile;
+    Stypes.dump (outputprefix ^ ".annot");
     raise e
 
 (* Compile a .ml file *)
@@ -79,13 +104,17 @@ let (++) x f = f x
 let (+++) (x, y) f = (x, f y)
 
 let implementation ppf sourcefile outputprefix =
+  Location.input_name := sourcefile;
   init_path ();
   let modulename =
     String.capitalize(Filename.basename(chop_extension_if_any sourcefile)) in
+  check_unit_name ppf sourcefile modulename;
   Env.set_unit_name modulename;
   let inputfile = Pparse.preprocess sourcefile in
   let env = initial_env() in
   Compilenv.reset ?packname:!Clflags.for_package modulename;
+  let cmxfile = outputprefix ^ ".cmx" in
+  let objfile = outputprefix ^ ext_obj in
   try
     if !Clflags.print_types then ignore(
       Pparse.file ppf inputfile Parse.implementation ast_impl_magic_number
@@ -102,12 +131,16 @@ let implementation ppf sourcefile outputprefix =
       +++ Simplif.simplify_lambda
       +++ print_if ppf Clflags.dump_lambda Printlambda.lambda
       ++ Asmgen.compile_implementation outputprefix ppf;
-      Compilenv.save_unit_info (outputprefix ^ ".cmx");
+      Compilenv.save_unit_info cmxfile;
     end;
     Warnings.check_fatal ();
-    Pparse.remove_preprocessed inputfile
+    Pparse.remove_preprocessed inputfile;
+    Stypes.dump (outputprefix ^ ".annot");
   with x ->
+    remove_file objfile;
+    remove_file cmxfile;
     Pparse.remove_preprocessed_if_ast inputfile;
+    Stypes.dump (outputprefix ^ ".annot");
     raise x
 
 let c_file name =
