@@ -10,7 +10,7 @@
 ;(*                                                                     *)
 ;(***********************************************************************)
 
-;(* $Id: caml-help.el,v 1.16.4.1 2004/08/09 16:09:33 doligez Exp $ *)
+;(* $Id$ *)
 
 ;; caml-info.el --- contextual completion and help to caml-mode
 
@@ -27,27 +27,27 @@
 ;;   - dump some databaes: Info, Lib, ...
 ;;   - accept a search path for local libraries instead of current dir
 ;;     (then distinguish between different modules lying in different
-;;     directories) 
+;;     directories)
 ;;   - improve the construction for info files.
 ;;
-;;  Abstract over 
+;;  Abstract over
 ;;   - the viewing method and the database, so that the documentation for
-;;     and identifier could be search in 
+;;     and identifier could be search in
 ;;       * info / html / man / mli's sources
 ;;       * viewed in emacs or using an external previewer.
 ;;
 ;;  Take all identifiers (labels, Constructors, exceptions, etc.)
-;;       
+;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 
 (eval-and-compile
-  (if (and (boundp 'running-xemacs) running-xemacs) 
+  (if (and (boundp 'running-xemacs) running-xemacs)
       (require 'caml-xemacs)
     (require 'caml-emacs)))
 
 ;; Loading or building databases.
-;; 
+;;
 
 ;; variables to be customized
 
@@ -73,7 +73,7 @@
              "ocamlc -where")))))
     ocaml-lib-path)
 
-      
+
 
 ;; General purpose auxiliary functions
 
@@ -92,17 +92,17 @@
           (if (stringp path)
               (if (file-directory-p path) path nil)
             (mapconcat '(lambda (d) (if (file-directory-p d) d))
-                       path " "))) 
+                       path " ")))
          (command
           (and path-string
                (concat "find " path-string
                        " '(' " filter " ')' "
                        (if depth (concat " -maxdepth " (int-to-string depth)))
-                       (if split nil " -printf '%\p '") 
+                       (if split nil " -printf '%\p '")
                        )))
           (files
            (and command (shell-command-to-string command))))
-         (if (and split (stringp files)) (split-string files "\n") files) 
+         (if (and split (stringp files)) (split-string files "\n") files)
          ))
 
 ;; Specialized auxiliary functions
@@ -111,7 +111,7 @@
 ;; Global table of modules contents of modules loaded lazily.
 
 (defvar ocaml-module-alist 'lazy
-  "A-list of modules with how and where to find help information. 
+  "A-list of modules with how and where to find help information.
   'delay means non computed yet")
 
 (defun ocaml-add-mli-modules (modules tag &optional path)
@@ -209,7 +209,7 @@
       alist)
       ))
 
-;; Local list of visible modules. 
+;; Local list of visible modules.
 
 (defvar ocaml-visible-modules 'lazy
   "A-list of open modules, local to every file.")
@@ -249,8 +249,8 @@ When call interactively, make completion over known modules."
   (message "%S" (mapcar 'car (ocaml-visible-modules))))
 
 (defun ocaml-close-module (arg)
-  "*Close module of name ARG when ARG is a string. 
-When call interactively, make completion over visible modules. 
+  "*Close module of name ARG when ARG is a string.
+When call interactively, make completion over visible modules.
 Otherwise if ARG is true, close all modules and reset to default. "
   (interactive "P")
   (if (= (prefix-numeric-value arg) 4)
@@ -268,27 +268,27 @@ Otherwise if ARG is true, close all modules and reset to default. "
                        ocaml-visible-modules))
       ))
   (message "%S" (mapcar 'car (ocaml-visible-modules))))
-           
+
 
 ;; Look for identifiers around point
 
 (defun ocaml-qualified-identifier (&optional show)
-  "Search for a qualified identifier (Path. entry) around point. 
+  "Search for a qualified identifier (Path. entry) around point.
 
 Entry may be nil.
-Currently, the path may only be nil or a single Module. 
-For paths is of the form Module.Path', it returns Module 
-and always nil for entry. 
+Currently, the path may only be nil or a single Module.
+For paths is of the form Module.Path', it returns Module
+and always nil for entry.
 
-If defined Module and Entry are represented by a region in the buffer, 
-and are nil otherwise. 
+If defined Module and Entry are represented by a region in the buffer,
+and are nil otherwise.
 
-For debugging purposes, it returns the string Module.entry if called 
-with an optional non-nil argument. 
+For debugging purposes, it returns the string Module.entry if called
+with an optional non-nil argument.
 "
   (save-excursion
     (let ((module) (entry))
-      (if (looking-at "[ \n]") (skip-chars-backward " ")) 
+      (if (looking-at "[ \n]") (skip-chars-backward " "))
       (if (re-search-backward
            "\\([^A-Za-z0-9_.']\\|\\`\\)\\([A-Za-z0-9_']*[.]\\)*[A-Za-z0-9_']*\\="
            (- (point) 100) t)
@@ -314,7 +314,7 @@ with an optional non-nil argument.
   (let ((list
          (or
           (and module
-               (list 
+               (list
                 (or (assoc module (ocaml-module-alist))
                     (error "Unknown module %s" module))))
           (ocaml-visible-modules))))
@@ -333,19 +333,19 @@ with an optional non-nil argument.
       )))
 
 (defun caml-complete (arg)
-  "Does completion for OCaml identifiers qualified. 
+  "Does completion for OCaml identifiers qualified.
 
-It attemps to recognize an qualified identifier Module . entry 
+It attemps to recognize an qualified identifier Module . entry
 around point using function \\[ocaml-qualified-identifier].
 
 If Module is defined, it does completion for identifier in Module.
 
-If Module is undefined, it does completion in visible modules. 
-Then, if completion fails, it does completion among  all modules 
+If Module is undefined, it does completion in visible modules.
+Then, if completion fails, it does completion among  all modules
 where identifier is defined."
   (interactive "p")
   (let* ((module-entry (ocaml-qualified-identifier)) (entry)
-         (module) 
+         (module)
          (beg) (end) (pattern))
     (if (car module-entry)
         (progn
@@ -364,7 +364,7 @@ where identifier is defined."
                    (progn (setq entry (cdr module-entry)) t))
               (error "Unknown module %s" module))))
     (if (consp (cdr module-entry))
-        (progn         
+        (progn
           (setq beg (cadr module-entry))
           (setq end (cddr module-entry)))
       (if (and module
@@ -408,7 +408,7 @@ where identifier is defined."
                        (delete-region (caar module-entry) end)
                      (delete-region beg end))
                    (insert module "." pattern))))
-                     
+
               ((not (string-equal pattern completion))
                (delete-region beg end)
                (goto-char beg)
@@ -426,10 +426,10 @@ where identifier is defined."
 
 (defvar ocaml-info-prefix "ocaml-lib"
   "Prefix of ocaml info files describing library modules.
-Suffix .info will be added to info files. 
+Suffix .info will be added to info files.
 Additional suffix .gz may be added if info files are compressed.
 ")
-;; 
+;;
 
 (defun ocaml-hevea-info-add-entries (entries dir name)
   (let*
@@ -470,9 +470,9 @@ Additional suffix .gz may be added if info files are compressed.
     entries))
 
 (defun ocaml-hevea-info ()
-  "The default way to create an info data base from the value 
-of \\[Info-default-directory-list] and the base name \\[ocaml-info-name] 
-of files to look for. 
+  "The default way to create an info data base from the value
+of \\[Info-default-directory-list] and the base name \\[ocaml-info-name]
+of files to look for.
 
 This uses info files produced by HeVeA.
 "
@@ -497,7 +497,8 @@ This uses info files produced by HeVeA.
     (message "Scanning info files in %s" dir)
     (save-window-excursion
       (set-buffer (get-buffer-create "*caml-help*"))
-      (or (shell-command command (current-buffer)) (error "HERE"))
+      (or (shell-command command (current-buffer))
+          (error "Could not run:%s" command))
       (goto-char (point-min))
       (while (re-search-forward module-regexp (point-max) t)
         (if (equal (char-after (match-end 1)) 127)
@@ -512,8 +513,8 @@ This uses info files produced by HeVeA.
     entries))
 
 (defun ocaml-ocamldoc-info ()
-  "The default way to create an info data base from the value 
-of \\[Info-default-directory-list] and the base name \\[ocaml-info-name] 
+  "The default way to create an info data base from the value
+of \\[Info-default-directory-list] and the base name \\[ocaml-info-name]
 of files to look for.
 
 This uses info files produced by ocamldoc."
@@ -531,18 +532,18 @@ This uses info files produced by ocamldoc."
 ;; Continuing
 
 (defvar ocaml-info-alist 'ocaml-ocamldoc-info
-  "A-list binding module names to info entries: 
+  "A-list binding module names to info entries:
 
   nil means do not use info.
 
   A function to build the list lazily (at the first call). The result of
 the function call will be assign permanently to this variable for future
 uses. We provide two default functions \\[ocaml-info-default-function]
-(info produced by HeVeA is the default) and \\[ocaml-info-default-function] 
-(info produced by ocamldoc). 
+(info produced by HeVeA is the default) and \\[ocaml-info-default-function]
+(info produced by ocamldoc).
 
   Otherwise, this value should be an alist binding module names to info
-entries of the form to \"(entry)section\" be taken by the \\[info] 
+entries of the form to \"(entry)section\" be taken by the \\[info]
 command. An entry may be an info module or a complete file name."
 )
 
@@ -571,7 +572,7 @@ command. An entry may be an info module or a complete file name."
 (defun ocaml-buffer-substring (region)
   (and region (buffer-substring-no-properties (car region) (cdr region))))
 
-;; Help function. 
+;; Help function.
 
 
 (defun ocaml-goto-help (&optional module entry same-window)
@@ -588,7 +589,7 @@ current buffer using \\[ocaml-qualified-identifier]."
               (or (assoc module (ocaml-module-alist))
                   (and (file-exists-p
                         (concat (ocaml-uncapitalize module) ".mli"))
-                       (ocaml-get-or-make-module module))))                  
+                       (ocaml-get-or-make-module module))))
              (location (cdr (cadr module-info))))
         (cond
          (location
@@ -624,33 +625,35 @@ current buffer using \\[ocaml-qualified-identifier]."
             (progn
               (message "Help for entry %s not found in module %s"
                        entry module)
-              (goto-char here)))))
+              (goto-char here)))
+          ))
     (ocaml-link-activate (cdr info-section))
     (if (window-live-p window) (select-window window))
     ))
 
 (defun caml-help (arg)
-  "Find documentation for OCaml qualified identifiers. 
+  "Find documentation for OCaml qualified identifiers.
 
 It attemps to recognize an qualified identifier of the form
 ``Module . entry'' around point using function `ocaml-qualified-identifier'.
 
 If Module is undetermined it is temptatively guessed from the identifier name
-and according to visible modules. If this is still unsucessful,  the user is 
-then prompted for a Module name. 
+and according to visible modules. If this is still unsucessful,  the user is
+then prompted for a Module name.
 
 The documentation for Module is first seach in the info manual if available,
-then in the ``module.mli'' source file. The entry is then searched in the documentation. 
+then in the ``module.mli'' source file. The entry is then searched in the
+documentation.
 
-Visible modules are computed only once, at the first call. 
+Visible modules are computed only once, at the first call.
 Modules can be made visible explicitly with `ocaml-open-module' and
-hidden with `ocaml-close-module'. 
+hidden with `ocaml-close-module'.
 
 Prefix arg 0 forces recompilation of visible modules (and their content)
-from the file content. 
+from the file content.
 
 Prefix arg 4 prompts for Module and identifier instead of guessing values
-from the possition of point in the current buffer. 
+from the possition of point in the current buffer.
 "
   (interactive "p")
   (let ((module) (entry) (module-entry))
@@ -709,7 +712,7 @@ from the possition of point in the current buffer.
 (defvar ocaml-links nil
   "Local links in the current of last info node or interface file.
 
-The car of the list is a key that indentifies the module to prevent 
+The car of the list is a key that indentifies the module to prevent
 recompilation when next help command is relative to the same module.
 The cdr is a list of elments, each of which is an string and a pair of
 buffer positions."
@@ -717,7 +720,7 @@ buffer positions."
 (make-variable-buffer-local 'ocaml-links)
 
 (defun ocaml-info-links (section)
-  (cdr 
+  (cdr
    (if (and ocaml-links section (equal (car ocaml-links) section))
        ocaml-links
      (save-excursion
@@ -777,35 +780,36 @@ buffer positions."
                               (mapconcat 'car links "\\|")
                               "\\)[^A-Za-z0-9'_]"))
               (case-fold-search nil))
-          (goto-char (point-min))
-          (let ((buffer-read-only nil)
-                ;; use of dynamic scoping, need not be restored!
-                (modified-p (buffer-modified-p)))
-            (unwind-protect
-                (save-excursion
-                  (goto-char (point-min))
-                  (while (re-search-forward regexp (point-max) t)
-                    (put-text-property (match-beginning 1) (match-end 1)
-                                       'mouse-face 'highlight)
-                    (put-text-property (match-beginning 1) (match-end 1)
-                                       'local-map ocaml-link-map)
-                    (if (x-display-color-p)
-                        (put-text-property (match-beginning 1) (match-end 1)
-                                           'face 'ocaml-link-face)))
-                  )
-              ;; need to restore flag if buffer was unmodified. 
-              (unless modified-p (set-buffer-modified-p nil))
-              ))
-          ))))
+          (save-excursion
+            (goto-char (point-min))
+            (let ((buffer-read-only nil)
+                  ;; use of dynamic scoping, need not be restored!
+                  (modified-p (buffer-modified-p)))
+              (unwind-protect
+                  (save-excursion
+                    (goto-char (point-min))
+                    (while (re-search-forward regexp (point-max) t)
+                      (put-text-property (match-beginning 1) (match-end 1)
+                                         'mouse-face 'highlight)
+                      (put-text-property (match-beginning 1) (match-end 1)
+                                         'local-map ocaml-link-map)
+                      (if (x-display-color-p)
+                          (put-text-property (match-beginning 1) (match-end 1)
+                                             'face 'ocaml-link-face)))
+                    )
+                ;; need to restore flag if buffer was unmodified.
+                (unless modified-p (set-buffer-modified-p nil))
+                ))
+            )))))
+
 
-  
 
 ;; bindings ---now in caml.el
 
 ; (and
 ;  (boundp 'caml-mode-map)
 ;  (keymapp caml-mode-map)
-;  (progn 
+;  (progn
 ;    (define-key caml-mode-map [?\C-c?i] 'ocaml-add-path)
 ;    (define-key caml-mode-map [?\C-c?]] 'ocaml-close-module)
 ;    (define-key caml-mode-map [?\C-c?[] 'ocaml-open-module)
@@ -822,7 +826,7 @@ buffer positions."
 ;         (define-key map [open] '("Open module for help" . ocaml-open-module))
 ;         (define-key map [help] '("Help for identifier" . caml-help))
 ;         (define-key map [complete] '("Complete identifier" . caml-complete))
-;         ) 
+;         )
 ;    ))))
 
 
