@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: output.ml 9512 2010-01-07 15:15:07Z doligez $ *)
+(* $Id$ *)
 
 (* Output the DFA tables and its entry points *)
 
@@ -74,8 +74,8 @@ let output_tables oc tbl =
 
 let output_entry sourcefile ic oc oci e =
   let init_num, init_moves = e.auto_initial_state in
-  fprintf oc "%s %alexbuf =\n\
-  %a%a  __ocaml_lex_%s_rec %alexbuf %d\n"
+  fprintf oc "%s %alexbuf =\
+\n  %a%a  __ocaml_lex_%s_rec %alexbuf %d\n"
     e.auto_name
     output_args  e.auto_args
     (fun oc x ->
