@@ -1,4 +1,4 @@
-\" $Id: ocamlc.m 9111 2008-10-29 12:38:52Z doligez $
+\" $Id$
 
 .TH OCAMLC 1
 
@@ -390,9 +390,8 @@ specify the name of the output file produced.
 .B \-output\-obj
 Cause the linker to produce a C object file instead of a bytecode
 executable file. This is useful to wrap Caml code as a C library,
-callable from any C program. The name of the output object file is
-.B camlprog.o
-by default; it can be set with the
+callable from any C program. The name of the output object file 
+must be set with the
 .B \-o
 option. This
 option can also be used to produce a C source file (.c extension) or
@@ -421,9 +420,9 @@ as a preprocessor for each source file. The output of
 is redirected to
 an intermediate file, which is compiled. If there are no compilation
 errors, the intermediate file is deleted afterwards. The name of this
-file is built from the basename of the source file with the extension .ppi
-for an interface (.mli) file and .ppo for an implementation
-(.ml) file.
+file is built from the basename of the source file with the
+extension .ppi for an interface (.mli) file and .ppo for an
+implementation (.ml) file.
 .TP
 .B \-principal
 Check information path during type-checking, to make sure that all
@@ -476,7 +475,7 @@ invocations of the C compiler and linker in
 .B \-custom
 mode.  Useful to debug C library problems.
 .TP
-.B \-version
+.BR \-vnum or \-version
 Print the version number of the compiler in short form (e.g. "3.11.0"),
 then exit.
 .TP
@@ -486,85 +485,245 @@ VM-level threads library described in
 .IR The\ Objective\ Caml\ user's\ manual .
 .TP
 .BI \-w \ warning\-list
-Enable or disable warnings according to the argument
+Enable, disable, or mark as errors the warnings specified by the argument
 .IR warning\-list .
-The argument is a set of letters.  If a letter is
-uppercase, it enables the corresponding warnings; lowercase disables
-the warnings.  The correspondence is the following:
 
-.B A
-\ \ all warnings
+Each warning can be
+.IR enabled \ or\  disabled ,
+and each warning can be
+.I marked
+(as error) or
+.IR unmarked .
+If a warning is disabled, it isn't displayed and doesn't affect
+compilation in any way (even if it is marked).  If a warning is enabled,
+it is displayed normally by the compiler whenever the source code
+triggers it.  If it is enabled and marked, the compiler will stop with
+an error after displaying the warnings if the source code triggers it.
 
-.B C
-\ \ start of comments that look like mistakes
+The
+.I warning\-list
+argument is a sequence of warning specifiers, with no separators
+between them.  A warning specifier is one of the following:
 
-.B D
-\ \ use of deprecated features
+.BI + num
+\ \ Enable warning number
+.IR num .
 
-.B E
-\ \ fragile pattern matchings (matchings that will remain
+.BI \- num
+\ \ Disable warning number
+.IR num .
+
+.BI @ num
+\ \ Enable and mark warning number
+.IR num .
+
+.BI + letter
+\ \ Enable the set of warnings corresponding to
+.IR letter .
+The letter may be uppercase or lowercase.
+
+.BI \- letter
+\ \ Disable the set of warnings corresponding to
+.IR letter .
+The letter may be uppercase or lowercase.
+
+.BI @ letter
+\ \ Enable and mark the set of warnings corresponding to
+.IR letter .
+The letter may be uppercase or lowercase.
+
+.I uppercase\-letter
+\ \ Enable the set of warnings corresponding to
+.IR uppercase\-letter .
+
+.I lowercase\-letter
+\ \ Disable the set of warnings corresponding to
+.IR lowercase\-letter .
+
+The warning numbers are as follows.
+
+1
+\ \ \ Suspicious-looking start-of-comment mark.
+
+2
+\ \ \ Suspicious-looking end-of-comment mark.
+
+3
+\ \ \ Deprecated syntax.
+
+4
+\ \ \ Fragile pattern matching: matching that will remain
 complete even if additional constructors are added to one of the
-variant types matched)
+variant types matched.
 
-.B F
-\ \ partially applied functions (expressions whose result has
-function type and is ignored)
+5
+\ \ \ Partially applied function: expression whose result has
+function type and is ignored.
 
-.B L
-\ \ omission of labels in applications
+6
+\ \ \ Label omitted in function application.
 
-.B M
-\ \ overriding of methods
+7
+\ \ \ Some methods are overridden in the class where they are defined.
 
-.B P
-\ \ missing cases in pattern matchings (i.e. partial matchings)
+8
+\ \ \ Partial match: missing cases in pattern-matching.
 
-.B S
-\ \ expressions in the left-hand side of a sequence that don't
+9
+\ \ \ Missing fields in a record pattern.
+
+10
+\ \ Expression on the left-hand side of a sequence that doesn't
 have type
 .B unit
-(and that are not functions, see
+(and that is not a function, see warning number 5).
+
+11
+\ \ Redundant case in a pattern matching (unused match case).
+
+12
+\ \ Redundant sub-pattern in a pattern-matching.
+
+13
+\ \ Override of an instance variable.
+
+14
+\ \ Illegal backslash escape in a string constant.
+
+15
+\ \ Private method made public implicitly.
+
+16
+\ \ Unerasable optional argument.
+
+17
+\ \ Undeclared virtual method.
+
+18
+\ \ Non-principal type.
+
+19
+\ \ Type without principality.
+
+20
+\ \ Unused function argument.
+
+21
+\ \ Non-returning statement.
+
+22
+\ \ Camlp4 warning.
+
+23
+\ \ Useless record
+.B with
+clause.
+
+24
+\ \ Bad module name: the source file name is not a valid OCaml module name.
+
+25
+\ \ Pattern-matching with all clauses guarded.
+
+26
+\ \ Suspicious unused variable: unused variable that is bound with
+.BR let \ or \ as ,
+and doesn't start with an underscore (_) character.
+
+27
+\ \ Innocuous unused variable: unused variable that is not bound with
+.BR let \ nor \ as ,
+and doesn't start with an underscore (_) character.
+
+28
+\ \ A pattern contains a constant constructor applied to the underscore (_)
+pattern.
+
+29
+\ \ A non-escaped end-of-line was found in a string constant.  This may
+
+cause portability problems between Unix and Windows.
+
+The letters stand for the following sets of warnings.  Any letter not
+mentioned here corresponds to the empty set.
+
+.B A
+\ all warnings
+
+.B C
+\ 1, 2
+
+.B D
+\ 3
+
+.B E
+\ 4
+
 .B F
-above)
+\ 5
+
+.B L
+\ 6
+
+.B M
+\ 7
+
+.B P
+\ 8
+
+.B R
+\ 9
+
+.B S
+\ 10
 
 .B U
-\ \ redundant cases in pattern matching (unused cases)
+\ 11, 12
 
 .B V
-\ \ overriding of instance variables
-
-.B Y
-\ \ unused variables that are bound with
-.BR let \ or \ as ,
-and don't start with an underscore (_) character
-
-.B Z
-\ \ all other cases of unused variables that don't start with an
-underscore (_) character
+\ 13
 
 .B X
-\ \ warnings that don't fit in the above categories (except
-.BR A )
+\ 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25
+
+.B Y
+\ 26
+
+.B Z
+\ 27
+
 .IP
 The default setting is
-.BR \-w\ Aelz ,
-enabling all warnings except fragile
-pattern matchings, omitted labels, and innocuous unused variables.
+.BR \-w\ +a\-4\-6\-9\-27\-28\-29 .
 Note that warnings
-.BR F \ and \ S
+.BR 5 \ and \ 10
 are not always triggered, depending on the internals of the type checker.
 .TP
 .BI \-warn\-error \ warning\-list
-Turn the warnings indicated in the argument
-.I warning\-list
-into errors.  The compiler will stop with an error when one of these
+Mark as errors the warnings specified in the argument
+.IR warning\-list .
+The compiler will stop with an error when one of these
 warnings is emitted.  The
 .I warning\-list
 has the same meaning as for
-the "\-w" option: an uppercase character turns the corresponding
-warning into an error, a lowercase character leaves it as a warning.
+the
+.B \-w
+option: a
+.B +
+sign (or an uppercase letter) turns the corresponding warnings into errors, a
+.B \-
+sign (or a lowercase letter) turns them back into warnings, and a
+.B @
+sign both enables and marks the corresponding warnings.
+
+Note: it is not recommended to use warning sets (i.e. letters) as
+arguments to
+.B \-warn\-error
+in production code, because this can break your build when future versions
+of OCaml add some new warnings.
+
 The default setting is
-.B \-warn\-error\ a
+.B \-warn\-error\ +a
 (none of the warnings is treated as an error).
 .TP
 .B \-where
