@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc.ml,v 1.9.14.1 2009/04/09 13:56:38 guesdon Exp $ *)
+(* $Id$ *)
 
 (** Main module for bytecode. *)
 
@@ -53,14 +53,14 @@ let get_real_filename name =
      (
       let paths = Filename.current_dir_name :: paths @ [Odoc_config.custom_generators_path] in
       try
-	let d = List.find
-	    (fun d -> Sys.file_exists (Filename.concat d name))
-	    paths
-	in
-	Filename.concat d name
+        let d = List.find
+            (fun d -> Sys.file_exists (Filename.concat d name))
+            paths
+        in
+        Filename.concat d name
       with
-	Not_found ->
-	  failwith (M.file_not_found_in_paths paths name)
+        Not_found ->
+          failwith (M.file_not_found_in_paths paths name)
      )
 
 let _ =
@@ -81,11 +81,11 @@ let _ =
           prerr_endline (Odoc_messages.load_file_error file "Not_found");
           exit 1
       | Sys_error s
-      |	Failure s ->
+      | Failure s ->
           prerr_endline (Odoc_messages.load_file_error file s);
           exit 1
 
-let _ = print_DEBUG "Fin du chargement dynamique ventuel"
+let _ = print_DEBUG "Fin du chargement dynamique eventuel"
 
 let default_html_generator = new Odoc_html.html
 let default_latex_generator = new Odoc_latex.latex
@@ -145,6 +145,3 @@ let _ =
   )
   else
     exit 0
-
-
-(* eof $Id: odoc.ml,v 1.9.14.1 2009/04/09 13:56:38 guesdon Exp $ *)
