@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_analyse.ml,v 1.14 2006/04/16 23:28:21 doligez Exp $ *)
+(* $Id$ *)
 
 (** Analysis of source files. This module is strongly inspired from
     driver/main.ml :-) *)
@@ -158,38 +158,46 @@ module Sig_analyser = Odoc_sig.Analyser (Odoc_comments.Basic_info_retriever)
 let process_error exn =
   let report ppf = function
   | Lexer.Error(err, loc) ->
-      Location.print ppf loc;
+      Location.print_error ppf loc;
       Lexer.report_error ppf err
   | Syntaxerr.Error err ->
       Syntaxerr.report_error ppf err
   | Env.Error err ->
+      Location.print_error_cur_file ppf;
       Env.report_error ppf err
-  | Ctype.Tags(l, l') -> fprintf ppf
+  | Ctype.Tags(l, l') ->
+      Location.print_error_cur_file ppf;
+      fprintf ppf
       "In this program,@ variant constructors@ `%s and `%s@ \
        have the same hash value." l l'
   | Typecore.Error(loc, err) ->
-      Location.print ppf loc; Typecore.report_error ppf err
+      Location.print_error ppf loc; Typecore.report_error ppf err
   | Typetexp.Error(loc, err) ->
-      Location.print ppf loc; Typetexp.report_error ppf err
+      Location.print_error ppf loc; Typetexp.report_error ppf err
   | Typedecl.Error(loc, err) ->
-      Location.print ppf loc; Typedecl.report_error ppf err
+      Location.print_error ppf loc; Typedecl.report_error ppf err
   | Includemod.Error err ->
+      Location.print_error_cur_file ppf;
       Includemod.report_error ppf err
   | Typemod.Error(loc, err) ->
-      Location.print ppf loc; Typemod.report_error ppf err
+      Location.print_error ppf loc; Typemod.report_error ppf err
   | Translcore.Error(loc, err) ->
-      Location.print ppf loc; Translcore.report_error ppf err
+      Location.print_error ppf loc; Translcore.report_error ppf err
   | Sys_error msg ->
+      Location.print_error_cur_file ppf;
       fprintf ppf "I/O error: %s" msg
   | Typeclass.Error(loc, err) ->
-      Location.print ppf loc; Typeclass.report_error ppf err
+      Location.print_error ppf loc; Typeclass.report_error ppf err
   | Translclass.Error(loc, err) ->
-      Location.print ppf loc; Translclass.report_error ppf err
+      Location.print_error ppf loc; Translclass.report_error ppf err
   | Warnings.Errors (n) ->
-      fprintf ppf "@.Error: error-enabled warnings (%d occurrences)" n
+      Location.print_error_cur_file ppf;
+      fprintf ppf "Error-enabled warnings (%d occurrences)" n
   | x ->
       fprintf ppf "@]";
-      fprintf ppf "Compilation error. Use the OCaml compiler to get more details."
+      fprintf ppf
+        "Compilation error(%s). Use the OCaml compiler to get more details."
+        (Printexc.to_string x)
   in
   Format.fprintf Format.err_formatter "@[%a@]@." report exn
 
@@ -208,6 +216,7 @@ let process_file ppf sourcefile =
   match sourcefile with
     Odoc_args.Impl_file file ->
       (
+       Location.input_name := file;
        try
          let (parsetree_typedtree_opt, input_file) = process_implementation_file ppf file in
          match parsetree_typedtree_opt with
@@ -239,6 +248,7 @@ let process_file ppf sourcefile =
       )
   | Odoc_args.Intf_file file ->
       (
+       Location.input_name := file;
        try
          let (ast, signat, input_file) = process_interface_file ppf file in
          let file_module = Sig_analyser.analyse_signature file
@@ -266,6 +276,7 @@ let process_file ppf sourcefile =
            None
       )
   | Odoc_args.Text_file file ->
+      Location.input_name := file;
       try
         let mod_name =
           String.capitalize (Filename.basename (Filename.chop_extension file))
@@ -416,6 +427,8 @@ and remove_module_elements_between_stop_in_module_kind k =
   | Odoc_module.Module_constraint (k2, mtkind) ->
       Odoc_module.Module_constraint (remove_module_elements_between_stop_in_module_kind k2,
                          remove_module_elements_between_stop_in_module_type_kind mtkind)
+  | Odoc_module.Module_typeof _ -> k
+  | Odoc_module.Module_unpack _ -> k
 
 (** Remove the module elements between the stop special comment, in the given module type kind. *)
 and remove_module_elements_between_stop_in_module_type_kind tk =
@@ -426,7 +439,7 @@ and remove_module_elements_between_stop_in_module_type_kind tk =
   | Odoc_module.Module_type_alias _ -> tk
   | Odoc_module.Module_type_with (tk2, s) ->
       Odoc_module.Module_type_with (remove_module_elements_between_stop_in_module_type_kind tk2, s)
-
+  | Odoc_module.Module_type_typeof _ -> tk
 
 (** Remove elements between the stop special comment. *)
 let remove_elements_between_stop module_list =
