@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_ast.ml 9371 2009-10-16 12:40:04Z doligez $ *)
+(* $Id$ *)
 
 (** Analysis of implementation files. *)
 open Misc
@@ -514,17 +514,21 @@ module Analyser =
             in
             (acc_inher, acc_fields @ ele_comments)
 
-        | (Parsetree.Pcf_inher (p_clexp, _)) :: q  ->
+        | (Parsetree.Pcf_inher (_, p_clexp, _)) :: q  ->
             let tt_clexp =
               let n = List.length acc_inher in
               try Typedtree_search.get_nth_inherit_class_expr tt_cls n
-              with Not_found -> raise (Failure (Odoc_messages.inherit_classexp_not_found_in_typedtree n))
+              with Not_found ->
+                raise (Failure (
+                       Odoc_messages.inherit_classexp_not_found_in_typedtree n))
             in
             let (info_opt, ele_comments) =
               get_comments_in_class last_pos
                 p_clexp.Parsetree.pcl_loc.Location.loc_start.Lexing.pos_cnum
             in
-            let text_opt = match info_opt with None -> None | Some i -> i.Odoc_types.i_desc in
+            let text_opt =
+              match info_opt with None -> None
+              | Some i -> i.Odoc_types.i_desc in
             let name = tt_name_of_class_expr tt_clexp in
             let inher =
               {
@@ -537,8 +541,8 @@ module Analyser =
               p_clexp.Parsetree.pcl_loc.Location.loc_end.Lexing.pos_cnum
               q
 
-      | ((Parsetree.Pcf_val (label, mutable_flag, _, loc) |
-              Parsetree.Pcf_valvirt (label, mutable_flag, _, loc) ) as x) :: q ->
+      | ((Parsetree.Pcf_val (label, mutable_flag, _, _, loc) |
+          Parsetree.Pcf_valvirt (label, mutable_flag, _, loc) ) as x) :: q ->
             let virt = match x with Parsetree.Pcf_val _ -> false | _ -> true in
             let complete_name = Name.concat current_class_name label in
             let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
@@ -602,7 +606,7 @@ module Analyser =
 
             iter acc_inher (acc_fields @ ele_comments @ [ Class_method met ]) loc.Location.loc_end.Lexing.pos_cnum q
 
-        | (Parsetree.Pcf_meth  (label, private_flag, _, loc)) :: q ->
+        | (Parsetree.Pcf_meth  (label, private_flag, _, _, loc)) :: q ->
             let complete_name = Name.concat current_class_name label in
             let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
             let exp =
@@ -846,7 +850,8 @@ module Analyser =
       | Typedtree.Tmod_constraint (m_exp, _, _) -> tt_name_from_module_expr m_exp
       | Typedtree.Tmod_structure _
       | Typedtree.Tmod_functor _
-      | Typedtree.Tmod_apply _ ->
+      | Typedtree.Tmod_apply _
+      | Typedtree.Tmod_unpack _ ->
           Odoc_messages.struct_end
 
     (** Get the list of included modules in a module structure of a typed tree. *)
@@ -1642,6 +1647,24 @@ module Analyser =
             m_kind = Module_struct elements2 ;
           }
 
+      | (Parsetree.Pmod_unpack (p_exp, pkg_type),
+         Typedtree.Tmod_unpack (t_exp, tt_modtype)) ->
+          print_DEBUG ("Odoc_ast: case Parsetree.Pmod_unpack + Typedtree.Tmod_unpack "^module_name);
+          let code =
+            let loc = p_module_expr.Parsetree.pmod_loc in
+            let loc_end = loc.Location.loc_end.Lexing.pos_cnum in
+            let exp_loc = p_exp.Parsetree.pexp_loc in
+            let exp_loc_end = exp_loc.Location.loc_end.Lexing.pos_cnum in
+            let s = get_string_of_file exp_loc_end loc_end in
+            Printf.sprintf "(val ...%s" s
+          in
+          let name = Odoc_env.full_module_type_name env (Name.from_longident (fst pkg_type)) in
+          let alias = { mta_name = name ; mta_module = None } in
+          { m_base with
+            m_type = Odoc_env.subst_module_type env tt_modtype ;
+            m_kind = Module_unpack (code, alias) ;
+          }
+
       | (parsetree, typedtree) ->
           (*DEBUG*)let s_parse =
           (*DEBUG*)  match parsetree with
@@ -1650,6 +1673,7 @@ module Analyser =
           (*DEBUG*)  | Parsetree.Pmod_functor _ -> "Pmod_functor"
           (*DEBUG*)  | Parsetree.Pmod_apply _ -> "Pmod_apply"
           (*DEBUG*)  | Parsetree.Pmod_constraint _ -> "Pmod_constraint"
+          (*DEBUG*)  | Parsetree.Pmod_unpack _ -> "Pmod_unpack"
           (*DEBUG*)in
           (*DEBUG*)let s_typed =
           (*DEBUG*)  match typedtree with
@@ -1658,6 +1682,7 @@ module Analyser =
           (*DEBUG*)  | Typedtree.Tmod_functor _ -> "Tmod_functor"
           (*DEBUG*)  | Typedtree.Tmod_apply _ -> "Tmod_apply"
           (*DEBUG*)  | Typedtree.Tmod_constraint _ -> "Tmod_constraint"
+          (*DEBUG*)  | Typedtree.Tmod_unpack _ -> "Tmod_unpack"
           (*DEBUG*)in
           (*DEBUG*)let code = get_string_of_file pos_start pos_end in
           print_DEBUG (Printf.sprintf "code=%s\ns_parse=%s\ns_typed=%s\n" code s_parse s_typed);
