@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_cross.ml,v 1.17 2006/09/20 11:14:36 doligez Exp $ *)
+(* $Id$ *)
 
 (** Cross referencing. *)
 
@@ -244,28 +244,28 @@ let lookup_exception name =
 class scan =
   object
     inherit Odoc_scan.scanner
-    method scan_value v =
+    method! scan_value v =
       add_known_element v.val_name (Odoc_search.Res_value v)
-    method scan_type t =
+    method! scan_type t =
       add_known_element t.ty_name (Odoc_search.Res_type t)
-    method scan_exception e =
+    method! scan_exception e =
       add_known_element e.ex_name (Odoc_search.Res_exception e)
-    method scan_attribute a =
+    method! scan_attribute a =
       add_known_element a.att_value.val_name
         (Odoc_search.Res_attribute a)
-    method scan_method m =
+    method! scan_method m =
       add_known_element m.met_value.val_name
         (Odoc_search.Res_method m)
-    method scan_class_pre c =
+    method! scan_class_pre c =
       add_known_element c.cl_name (Odoc_search.Res_class c);
       true
-    method scan_class_type_pre c =
+    method! scan_class_type_pre c =
       add_known_element c.clt_name (Odoc_search.Res_class_type c);
       true
-    method scan_module_pre m =
+    method! scan_module_pre m =
       add_known_element m.m_name (Odoc_search.Res_module m);
       true
-    method scan_module_type_pre m =
+    method! scan_module_type_pre m =
       add_known_element m.mt_name (Odoc_search.Res_module_type m);
       true
 
@@ -343,6 +343,34 @@ let rec associate_in_module module_list (acc_b_modif, acc_incomplete_top_module_
           { mt_name = "" ; mt_info = None ; mt_type = None ;
             mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
             mt_loc = Odoc_types.dummy_loc }
+
+     | Module_typeof _ ->
+        (acc_b, acc_inc, acc_names)
+
+     | Module_unpack (code, mta) ->
+        begin
+          match mta.mta_module with
+            Some _ ->
+              (acc_b, acc_inc, acc_names)
+          | None ->
+              let mt_opt =
+                try Some (lookup_module_type mta.mta_name)
+                with Not_found -> None
+              in
+              match mt_opt with
+                None -> (acc_b, (Name.head m.m_name) :: acc_inc,
+                   (* we don't want to output warning messages for
+                      "sig ... end" or "struct ... end" modules not found *)
+                   (if mta.mta_name = Odoc_messages.struct_end or
+                      mta.mta_name = Odoc_messages.sig_end then
+                      acc_names
+                    else
+                      (NF_mt mta.mta_name) :: acc_names)
+                  )
+              | Some mt ->
+                  mta.mta_module <- Some mt ;
+                  (true, acc_inc, acc_names)
+        end
   in
   iter_kind (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) m.m_kind
 
@@ -362,27 +390,31 @@ and associate_in_module_type module_list (acc_b_modif, acc_incomplete_top_module
         iter_kind (acc_b, acc_inc, acc_names) k
 
     | Module_type_alias mta ->
-        match mta.mta_module with
-           Some _ ->
-             (acc_b, acc_inc, acc_names)
-         | None ->
-             let mt_opt =
-               try Some (lookup_module_type mta.mta_name)
-               with Not_found -> None
-             in
-             match mt_opt with
-               None -> (acc_b, (Name.head mt.mt_name) :: acc_inc,
-                        (* we don't want to output warning messages for
-                           "sig ... end" or "struct ... end" modules not found *)
-                        (if mta.mta_name = Odoc_messages.struct_end or
-                          mta.mta_name = Odoc_messages.sig_end then
-                          acc_names
-                        else
-                          (NF_mt mta.mta_name) :: acc_names)
-                       )
-             | Some mt ->
-                 mta.mta_module <- Some mt ;
-                 (true, acc_inc, acc_names)
+        begin
+          match mta.mta_module with
+            Some _ ->
+              (acc_b, acc_inc, acc_names)
+          | None ->
+              let mt_opt =
+                try Some (lookup_module_type mta.mta_name)
+                with Not_found -> None
+              in
+              match mt_opt with
+                None -> (acc_b, (Name.head mt.mt_name) :: acc_inc,
+                   (* we don't want to output warning messages for
+                      "sig ... end" or "struct ... end" modules not found *)
+                   (if mta.mta_name = Odoc_messages.struct_end or
+                      mta.mta_name = Odoc_messages.sig_end then
+                      acc_names
+                    else
+                      (NF_mt mta.mta_name) :: acc_names)
+                  )
+              | Some mt ->
+                  mta.mta_module <- Some mt ;
+                  (true, acc_inc, acc_names)
+        end
+    | Module_type_typeof _ ->
+        (acc_b, acc_inc, acc_names)
   in
   match mt.mt_kind with
     None -> (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found)
@@ -611,9 +643,10 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
   | Subscript t -> Subscript (assoc_comments_text parent_name module_list t)
   | Title (n, l_opt, t) -> Title (n, l_opt, (assoc_comments_text parent_name module_list t))
   | Link (s, t) -> Link (s, (assoc_comments_text parent_name module_list t))
-  | Ref (initial_name, None) ->
+  | Ref (initial_name, None, text_option) ->
       (
        let rec iter_parent ?parent_name name =
+         let name = Odoc_name.normalize_name name in
          let res =
            match get_known_elements name with
              [] ->
@@ -647,12 +680,12 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
                (name, Some kind)
          in
          match res with
-         | (name, Some k) -> Ref (name, Some k)
+         | (name, Some k) -> Ref (name, Some k, text_option)
          | (_, None) ->
              match parent_name with
                None ->
                  Odoc_messages.pwarning (Odoc_messages.cross_element_not_found initial_name);
-                 Ref (initial_name, None)
+                 Ref (initial_name, None, text_option)
              | Some p ->
                  let parent_name =
                    match Name.father p with
@@ -663,12 +696,12 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
        in
        iter_parent ~parent_name initial_name
       )
-  | Ref (initial_name, Some kind) ->
+  | Ref (initial_name, Some kind, text_option) ->
       (
        let rec iter_parent ?parent_name name =
          let v = (name, Some kind) in
          if was_verified v then
-           Ref (name, Some kind)
+           Ref (name, Some kind, text_option)
          else
            let res =
              match kind with
@@ -708,12 +741,12 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
                    (name, None)
            in
            match res with
-           | (name, Some k) -> Ref (name, Some k)
+           | (name, Some k) -> Ref (name, Some k, text_option)
            | (_, None) ->
                match parent_name with
                  None ->
                    Odoc_messages.pwarning (not_found_of_kind kind initial_name);
-                   Ref (initial_name, None)
+                   Ref (initial_name, None, text_option)
                | Some p ->
                    let parent_name =
                      match Name.father p with
@@ -729,6 +762,7 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
   | Index_list ->
       Index_list
   | Custom (s,t) -> Custom (s, (assoc_comments_text parent_name module_list t))
+  | Target (target, code) -> Target (target, code)
 
 and assoc_comments_text parent_name module_list text =
   List.map (assoc_comments_text_elements parent_name module_list) text
@@ -794,6 +828,8 @@ and assoc_comments_module_kind parent_name module_list mk =
       Module_constraint
         (assoc_comments_module_kind parent_name module_list mk1,
          assoc_comments_module_type_kind parent_name module_list mtk)
+  | Module_typeof _ -> mk
+  | Module_unpack _ -> mk
 
 and assoc_comments_module_type_kind parent_name module_list mtk =
   match mtk with
@@ -808,6 +844,7 @@ and assoc_comments_module_type_kind parent_name module_list mtk =
   | Module_type_with (mtk1, s) ->
       Module_type_with
         (assoc_comments_module_type_kind parent_name module_list mtk1, s)
+  | Module_type_typeof _ -> mtk
 
 and assoc_comments_class_kind parent_name module_list ck =
   match ck with
@@ -889,11 +926,11 @@ and assoc_comments_type module_list t =
   t.ty_info <- ao (assoc_comments_info parent module_list) t.ty_info ;
   (match t.ty_kind with
     Type_abstract -> ()
-  | Type_variant (vl, _) ->
+  | Type_variant vl ->
       List.iter
         (fun vc -> vc.vc_text <- ao (assoc_comments_text parent module_list) vc.vc_text)
         vl
-  | Type_record (fl, _) ->
+  | Type_record fl ->
       List.iter
         (fun rf -> rf.rf_text <- ao (assoc_comments_text parent module_list) rf.rf_text)
         fl
