@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_misc.ml 7470 2006-07-06 07:19:06Z pouillar $ *)
+(* $Id$ *)
 
 let no_blanks s =
   let len = String.length s in
@@ -124,7 +124,9 @@ let rec string_of_text t =
       | Odoc_types.Latex s -> "{% "^s^" %}"
       | Odoc_types.Link (s, t) ->
           "["^s^"]"^(string_of_text t)
-      | Odoc_types.Ref (name, _) ->
+      | Odoc_types.Ref (name, _, Some text) ->
+          Printf.sprintf "[%s]" (string_of_text text)
+      | Odoc_types.Ref (name, _, None) ->
           iter (Odoc_types.Code name)
       | Odoc_types.Superscript t ->
           "^{"^(string_of_text t)^"}"
@@ -138,6 +140,7 @@ let rec string_of_text t =
       | Odoc_types.Index_list ->
           ""
       | Odoc_types.Custom (_, t) -> string_of_text t
+      | Odoc_types.Target _ -> ""
   in
   String.concat "" (List.map iter t)
 
@@ -258,7 +261,8 @@ let rec text_no_title_no_list t =
     | Odoc_types.Code _
     | Odoc_types.CodePre _
     | Odoc_types.Verbatim _
-    | Odoc_types.Ref _ -> [t_ele]
+    | Odoc_types.Ref _
+    | Odoc_types.Target _ -> [t_ele]
     | Odoc_types.Newline ->  [Odoc_types.Newline]
     | Odoc_types.Block t -> [Odoc_types.Block (text_no_title_no_list t)]
     | Odoc_types.Bold t -> [Odoc_types.Bold (text_no_title_no_list t)]
@@ -274,7 +278,7 @@ let rec text_no_title_no_list t =
     | Odoc_types.Module_list l ->
         list_concat (Odoc_types.Raw ", ")
           (List.map
-             (fun s -> Odoc_types.Ref (s, Some Odoc_types.RK_module))
+             (fun s -> Odoc_types.Ref (s, Some Odoc_types.RK_module, None))
              l
           )
     | Odoc_types.Index_list -> []
@@ -309,6 +313,7 @@ let get_titles_in_text t =
     | Odoc_types.Module_list _ -> ()
     | Odoc_types.Index_list -> ()
     | Odoc_types.Custom (_, t) -> iter_text t
+    | Odoc_types.Target _ -> ()
   and iter_text te =
     List.iter iter_ele te
   in
@@ -400,7 +405,9 @@ and first_sentence_text_ele text_ele =
   | Odoc_types.Subscript _
   | Odoc_types.Module_list _
   | Odoc_types.Index_list -> (false, text_ele, None)
-  | Odoc_types.Custom _ -> (false, text_ele, None)
+  | Odoc_types.Custom _
+  | Odoc_types.Target _ -> (false, text_ele, None)
+
 
 let first_sentence_of_text t =
   let (_,t2,_) = first_sentence_text t in
@@ -479,7 +486,8 @@ let remove_option typ =
     | Types.Tobject _
     | Types.Tfield _
     | Types.Tnil
-    | Types.Tvariant _ -> t
+    | Types.Tvariant _
+    | Types.Tpackage _ -> t
     | Types.Tlink t2
     | Types.Tsubst t2 -> iter t2.Types.desc
   in
