@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_sig.ml,v 1.36.2.1 2005/11/10 14:44:36 guesdon Exp $ *)
+(* $Id$ *)
 
 (** Analysis of interface files. *)
 
@@ -107,7 +107,7 @@ module Signature_search =
       | _ -> assert false
 
     let search_attribute_type name class_sig =
-      let (_, type_expr) = Types.Vars.find name class_sig.Types.cty_vars in
+      let (_, _, type_expr) = Types.Vars.find name class_sig.Types.cty_vars in
       type_expr
 
     let search_method_type name class_sig =
@@ -172,32 +172,32 @@ module Analyser =
 
     let name_comment_from_type_kind pos_end pos_limit tk =
       match tk with
-        Parsetree.Ptype_abstract | Parsetree.Ptype_private ->
+        Parsetree.Ptype_abstract ->
           (0, [])
-      | Parsetree.Ptype_variant (cons_core_type_list_list, _) ->
+      | Parsetree.Ptype_variant cons_core_type_list_list ->
           let rec f acc cons_core_type_list_list =
             match cons_core_type_list_list with
               [] ->
                 (0, acc)
             | (name, core_type_list, loc) :: [] ->
                 let s = get_string_of_file
-		    loc.Location.loc_end.Lexing.pos_cnum
-		    pos_limit
-		in
+                    loc.Location.loc_end.Lexing.pos_cnum
+                    pos_limit
+                in
                 let (len, comment_opt) =  My_ir.just_after_special !file_name s in
                 (len, acc @ [ (name, comment_opt) ])
             | (name, core_type_list, loc) :: (name2, core_type_list2, loc2)
               :: q ->
-		let pos_end_first = loc.Location.loc_end.Lexing.pos_cnum in
-		let pos_start_second = loc2.Location.loc_start.Lexing.pos_cnum in
-		let s = get_string_of_file pos_end_first pos_start_second in
-		let (_,comment_opt) = My_ir.just_after_special !file_name  s in
-		f (acc @ [name, comment_opt])
+                let pos_end_first = loc.Location.loc_end.Lexing.pos_cnum in
+                let pos_start_second = loc2.Location.loc_start.Lexing.pos_cnum in
+                let s = get_string_of_file pos_end_first pos_start_second in
+                let (_,comment_opt) = My_ir.just_after_special !file_name  s in
+                f (acc @ [name, comment_opt])
                   ((name2, core_type_list2, loc2) :: q)
           in
           f [] cons_core_type_list_list
 
-      | Parsetree.Ptype_record (name_mutable_type_list, _) (* of (string * mutable_flag * core_type) list*) ->
+      | Parsetree.Ptype_record name_mutable_type_list (* of (string * mutable_flag * core_type) list*) ->
           let rec f = function
               [] ->
                 []
@@ -220,7 +220,7 @@ module Analyser =
         Types.Type_abstract ->
           Odoc_type.Type_abstract
 
-      | Types.Type_variant (l, priv) ->
+      | Types.Type_variant l ->
           let f (constructor_name, type_expr_list) =
             let comment_opt =
               try
@@ -235,9 +235,9 @@ module Analyser =
               vc_text = comment_opt
             }
           in
-          Odoc_type.Type_variant (List.map f l, priv = Asttypes.Private)
+          Odoc_type.Type_variant (List.map f l)
 
-      | Types.Type_record (l, _, priv) ->
+      | Types.Type_record (l, _) ->
           let f (field_name, mutable_flag, type_expr) =
             let comment_opt =
               try
@@ -253,23 +253,18 @@ module Analyser =
               rf_text = comment_opt
             }
           in
-          Odoc_type.Type_record (List.map f l, priv = Asttypes.Private)
+          Odoc_type.Type_record (List.map f l)
 
     (** Analysis of the elements of a class, from the information in the parsetree and in the class
        signature. @return the couple (inherited_class list, elements).*)
     let analyse_class_elements env current_class_name last_pos pos_limit
         class_type_field_list class_signature =
-      print_DEBUG "Types.Tcty_signature class_signature";
-      let f_DEBUG var (mutable_flag, type_exp) = print_DEBUG var in
-      Types.Vars.iter f_DEBUG class_signature.Types.cty_vars;
-      print_DEBUG ("Type de la classe "^current_class_name^" : ");
-      print_DEBUG (Odoc_print.string_of_type_expr class_signature.Types.cty_self);
       let get_pos_limit2 q =
         match q with
           [] -> pos_limit
         | ele2 :: _ ->
             match ele2 with
-              Parsetree.Pctf_val (_, _, _, loc)
+              Parsetree.Pctf_val (_, _, _, _, loc)
             | Parsetree.Pctf_virt (_, _, _, loc)
             | Parsetree.Pctf_meth (_, _, _, loc)
             | Parsetree.Pctf_cstr (_, _, loc) -> loc.Location.loc_start.Lexing.pos_cnum
@@ -330,7 +325,7 @@ module Analyser =
             in
             ([], ele_comments)
 
-        | Parsetree.Pctf_val (name, mutable_flag, _, loc) :: q ->
+        | Parsetree.Pctf_val (name, mutable_flag, virtual_flag, _, loc) :: q ->
             (* of (string * mutable_flag * core_type option * Location.t)*)
             let (comment_opt, eles_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
             let complete_name = Name.concat current_class_name name in
@@ -353,6 +348,7 @@ module Analyser =
                   val_loc = { loc_impl = None ; loc_inter = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum)} ;
                 } ;
                 att_mutable = mutable_flag = Asttypes.Mutable ;
+                att_virtual = virtual_flag = Asttypes.Virtual ;
               }
             in
             let pos_limit2 = get_pos_limit2 q in
@@ -531,8 +527,8 @@ module Analyser =
                 ex_args = List.map (Odoc_env.subst_type env) types_excep_decl ;
                 ex_alias = None ;
                 ex_loc = { loc_impl = None ; loc_inter = Some (!file_name, pos_start_ele) } ;
-		ex_code =
-		   (
+                ex_code =
+                   (
                     if !Odoc_args.keep_code then
                       Some (get_string_of_file pos_start_ele pos_end_ele)
                     else
@@ -595,7 +591,7 @@ module Analyser =
                   in
                   (* get the type kind with the associated comments *)
                   let type_kind = get_type_kind new_env name_comment_list sig_type_decl.Types.type_kind in
-		  let loc_start = type_decl.Parsetree.ptype_loc.Location.loc_start.Lexing.pos_cnum in
+                  let loc_start = type_decl.Parsetree.ptype_loc.Location.loc_start.Lexing.pos_cnum in
                   let new_end = type_decl.Parsetree.ptype_loc.Location.loc_end.Lexing.pos_cnum + maybe_more in
                   (* associate the comments to each constructor and build the [Type.t_type] *)
                   let new_type =
@@ -603,13 +599,14 @@ module Analyser =
                       ty_name = Name.concat current_module_name name ;
                       ty_info = assoc_com ;
                       ty_parameters =
-		        List.map2 (fun p (co,cn,_) ->
-			             (Odoc_env.subst_type new_env p,
-				      co, cn)
-				  )
-			sig_type_decl.Types.type_params
-			sig_type_decl.Types.type_variance;
-                      ty_kind = type_kind ;
+                        List.map2 (fun p (co,cn,_) ->
+                                     (Odoc_env.subst_type new_env p,
+                                      co, cn)
+                                  )
+                        sig_type_decl.Types.type_params
+                        sig_type_decl.Types.type_variance;
+                      ty_kind = type_kind;
+                      ty_private = sig_type_decl.Types.type_private;
                       ty_manifest =
                       (match sig_type_decl.Types.type_manifest with
                         None -> None
@@ -619,12 +616,12 @@ module Analyser =
                         loc_inter = Some (!file_name,loc_start) ;
                       };
                       ty_code =
-		        (
-			 if !Odoc_args.keep_code then
-			   Some (get_string_of_file loc_start new_end)
-			 else
-			   None
-			) ;
+                        (
+                         if !Odoc_args.keep_code then
+                           Some (get_string_of_file loc_start new_end)
+                         else
+                           None
+                        ) ;
                     }
                   in
                   let (maybe_more2, info_after_opt) =
@@ -662,15 +659,15 @@ module Analyser =
                 raise (Failure (Odoc_messages.module_not_found current_module_name name))
             in
             let module_kind = analyse_module_kind env complete_name module_type sig_module_type in
-	    let code_intf =
-	      if !Odoc_args.keep_code then
-		let loc = module_type.Parsetree.pmty_loc in
-		let st = loc.Location.loc_start.Lexing.pos_cnum in
-		let en = loc.Location.loc_end.Lexing.pos_cnum in
-		Some (get_string_of_file st en)
-	      else
-		None
-	    in
+            let code_intf =
+              if !Odoc_args.keep_code then
+                let loc = module_type.Parsetree.pmty_loc in
+                let st = loc.Location.loc_start.Lexing.pos_cnum in
+                let en = loc.Location.loc_end.Lexing.pos_cnum in
+                Some (get_string_of_file st en)
+              else
+                None
+            in
             let new_module =
               {
                 m_name = complete_name ;
@@ -681,9 +678,9 @@ module Analyser =
                 m_kind = module_kind ;
                 m_loc = { loc_impl = None ; loc_inter = Some (!file_name, pos_start_ele) } ;
                 m_top_deps = [] ;
-		m_code = None ;
-		m_code_intf = code_intf ;
-		m_text_only = false ;
+                m_code = None ;
+                m_code_intf = code_intf ;
+                m_text_only = false ;
               }
             in
             let (maybe_more, info_after_opt) =
@@ -701,7 +698,7 @@ module Analyser =
             (maybe_more, new_env2, [ Element_module new_module ])
 
         | Parsetree.Psig_recmodule decls ->
-	    (* we start by extending the environment *)
+            (* we start by extending the environment *)
             let new_env =
               List.fold_left
                 (fun acc_env -> fun (name, _) ->
@@ -713,13 +710,13 @@ module Analyser =
                     with Not_found ->
                       raise (Failure (Odoc_messages.module_not_found current_module_name name))
                   in
-		  match sig_module_type with
+                  match sig_module_type with
                     (* A VOIR : cela peut-il tre Tmty_ident ? dans ce cas, on aurait pas la signature *)
                     Types.Tmty_signature s ->
-		      Odoc_env.add_signature e complete_name ~rel: name s
-		  | _ ->
-		      print_DEBUG "not a Tmty_signature";
-		      e
+                      Odoc_env.add_signature e complete_name ~rel: name s
+                  | _ ->
+                      print_DEBUG "not a Tmty_signature";
+                      e
                 )
                 env
                 decls
@@ -729,8 +726,8 @@ module Analyser =
                 [] ->
                   (acc_maybe_more, [])
               | (name, modtype) :: q ->
-		  let complete_name = Name.concat current_module_name name in
-		  let loc_start = modtype.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
+                  let complete_name = Name.concat current_module_name name in
+                  let loc_start = modtype.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
                   let loc_end = modtype.Parsetree.pmty_loc.Location.loc_end.Lexing.pos_cnum in
                   let (assoc_com, ele_comments) =
                     if first then
@@ -738,7 +735,7 @@ module Analyser =
                     else
                       get_comments_in_module
                         last_pos
-			loc_start
+                        loc_start
                   in
                   let pos_limit2 =
                     match q with
@@ -752,18 +749,18 @@ module Analyser =
                       raise (Failure (Odoc_messages.module_not_found current_module_name name))
                   in
                   (* associate the comments to each constructor and build the [Type.t_type] *)
-		  let module_kind = analyse_module_kind new_env complete_name modtype sig_module_type in
-		  let code_intf =
-		    if !Odoc_args.keep_code then
-		      let loc = modtype.Parsetree.pmty_loc in
-		      let st = loc.Location.loc_start.Lexing.pos_cnum in
-		      let en = loc.Location.loc_end.Lexing.pos_cnum in
-		      Some (get_string_of_file st en)
-		    else
-		      None
-		  in
-		  let new_module =
-		    {
+                  let module_kind = analyse_module_kind new_env complete_name modtype sig_module_type in
+                  let code_intf =
+                    if !Odoc_args.keep_code then
+                      let loc = modtype.Parsetree.pmty_loc in
+                      let st = loc.Location.loc_start.Lexing.pos_cnum in
+                      let en = loc.Location.loc_end.Lexing.pos_cnum in
+                      Some (get_string_of_file st en)
+                    else
+                      None
+                  in
+                  let new_module =
+                    {
                       m_name = complete_name ;
                       m_type = sig_module_type;
                       m_info = assoc_com ;
@@ -772,17 +769,17 @@ module Analyser =
                       m_kind = module_kind ;
                       m_loc = { loc_impl = None ; loc_inter = Some (!file_name, pos_start_ele) } ;
                       m_top_deps = [] ;
-		      m_code = None ;
-		      m_code_intf = code_intf ;
-		      m_text_only = false ;
-		    }
-		  in
-		  let (maybe_more, info_after_opt) =
-		    My_ir.just_after_special
+                      m_code = None ;
+                      m_code_intf = code_intf ;
+                      m_text_only = false ;
+                    }
+                  in
+                  let (maybe_more, info_after_opt) =
+                    My_ir.just_after_special
                       !file_name
                       (get_string_of_file loc_end pos_limit2)
-		  in
-		  new_module.m_info <- merge_infos new_module.m_info info_after_opt ;
+                  in
+                  new_module.m_info <- merge_infos new_module.m_info info_after_opt ;
 
                   let (maybe_more2, eles) = f
                       maybe_more
@@ -794,13 +791,22 @@ module Analyser =
             let (maybe_more, mods) = f ~first: true 0 pos_start_ele decls in
             (maybe_more, new_env, mods)
 
-        | Parsetree.Psig_modtype (name, Parsetree.Pmodtype_abstract) ->
+        | Parsetree.Psig_modtype (name, pmodtype_decl) ->
+            let complete_name = Name.concat current_module_name name in
             let sig_mtype =
               try Signature_search.search_module_type table name
               with Not_found ->
                 raise (Failure (Odoc_messages.module_type_not_found current_module_name name))
             in
-            let complete_name = Name.concat current_module_name name in
+            let module_type_kind =
+              match pmodtype_decl with
+                Parsetree.Pmodtype_abstract -> None
+              | Parsetree.Pmodtype_manifest module_type ->
+                match sig_mtype with
+                | Some sig_mtype -> Some (analyse_module_type_kind env complete_name module_type sig_mtype)
+                | None -> None
+            in
+
             let mt =
               {
                 mt_name = complete_name ;
@@ -808,38 +814,6 @@ module Analyser =
                 mt_type = sig_mtype ;
                 mt_is_interface = true ;
                 mt_file = !file_name ;
-                mt_kind = None ;
-                mt_loc = { loc_impl = None ; loc_inter = Some (!file_name, pos_start_ele) } ;
-              }
-            in
-            let (maybe_more, info_after_opt) =
-              My_ir.just_after_special
-                !file_name
-                (get_string_of_file pos_end_ele pos_limit)
-            in
-            mt.mt_info <- merge_infos mt.mt_info info_after_opt ;
-            let new_env = Odoc_env.add_module_type env mt.mt_name in
-            (maybe_more, new_env, [ Element_module_type mt ])
-
-        | Parsetree.Psig_modtype (name, Parsetree.Pmodtype_manifest module_type) ->
-            let complete_name = Name.concat current_module_name name in
-            let sig_mtype_opt =
-              try Signature_search.search_module_type table name
-              with Not_found ->
-                raise (Failure (Odoc_messages.module_type_not_found current_module_name name))
-            in
-            let module_type_kind =
-              match sig_mtype_opt with
-              | Some sig_mtype -> Some (analyse_module_type_kind env complete_name module_type sig_mtype)
-              | None -> None
-            in
-            let mt =
-              {
-                mt_name = complete_name ;
-                mt_info = comment_opt ;
-                mt_type = sig_mtype_opt ;
-                mt_is_interface = true ;
-                mt_file = !file_name ;
                 mt_kind = module_type_kind ;
                 mt_loc = { loc_impl = None ; loc_inter = Some (!file_name, pos_start_ele) } ;
               }
@@ -852,7 +826,7 @@ module Analyser =
             mt.mt_info <- merge_infos mt.mt_info info_after_opt ;
             let new_env = Odoc_env.add_module_type env mt.mt_name in
             let new_env2 =
-              match sig_mtype_opt with (* A VOIR : cela peut-il tre Tmty_ident ? dans ce cas, on aurait pas la signature *)
+              match sig_mtype with (* A VOIR : cela peut-il tre Tmty_ident ? dans ce cas, on aurait pas la signature *)
                 Some (Types.Tmty_signature s) -> Odoc_env.add_signature new_env mt.mt_name ~rel: (Name.simple mt.mt_name) s
               | _ -> new_env
             in
@@ -868,14 +842,18 @@ module Analyser =
                   "??"
               | Parsetree.Pmty_with (mt, _) ->
                   f mt.Parsetree.pmty_desc
+              | Parsetree.Pmty_typeof mexpr ->
+                  match mexpr.Parsetree.pmod_desc with
+                    Parsetree.Pmod_ident longident -> Name.from_longident longident
+                  | _ -> "??"
             in
-	    let name = (f module_type.Parsetree.pmty_desc) in
-	    let full_name = Odoc_env.full_module_or_module_type_name env name in
+            let name = f module_type.Parsetree.pmty_desc in
+            let full_name = Odoc_env.full_module_or_module_type_name env name in
             let im =
               {
                 im_name = full_name ;
                 im_module = None ;
-		im_info = comment_opt;
+                im_info = comment_opt;
               }
             in
             (0, env, [ Element_included_module im ]) (* A VOIR : tendre l'environnement ? avec quoi ? *)
@@ -1057,28 +1035,28 @@ module Analyser =
 
       | Parsetree.Pmty_functor (_,pmodule_type2, module_type2) ->
           (
-	   let loc_start = pmodule_type2.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
+           let loc_start = pmodule_type2.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
            let loc_end = pmodule_type2.Parsetree.pmty_loc.Location.loc_end.Lexing.pos_cnum in
-	   let mp_type_code = get_string_of_file loc_start loc_end in
-	   print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
+           let mp_type_code = get_string_of_file loc_start loc_end in
+           print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
            match sig_module_type with
              Types.Tmty_functor (ident, param_module_type, body_module_type) ->
-	       let mp_kind = analyse_module_type_kind env
-		   current_module_name pmodule_type2 param_module_type
-	       in
+               let mp_kind = analyse_module_type_kind env
+                   current_module_name pmodule_type2 param_module_type
+               in
                let param =
                  {
                    mp_name = Name.from_ident ident ;
                    mp_type = Odoc_env.subst_module_type env param_module_type ;
-		   mp_type_code = mp_type_code ;
-		   mp_kind = mp_kind ;
+                   mp_type_code = mp_type_code ;
+                   mp_kind = mp_kind ;
                  }
                in
-	       let k = analyse_module_type_kind env
-		   current_module_name
-		   module_type2
-		   body_module_type
-	       in
+               let k = analyse_module_type_kind env
+                   current_module_name
+                   module_type2
+                   body_module_type
+               in
                Module_type_functor (param, k)
 
            | _ ->
@@ -1096,11 +1074,17 @@ module Analyser =
            Module_type_with (k, s)
           )
 
-    (** Analyse of a Parsetree.module_type and a Types.module_type.*)
+      | Parsetree.Pmty_typeof module_expr ->
+          let loc_start = module_expr.Parsetree.pmod_loc.Location.loc_start.Lexing.pos_cnum in
+          let loc_end = module_expr.Parsetree.pmod_loc.Location.loc_end.Lexing.pos_cnum in
+          let s = get_string_of_file loc_start loc_end in
+          Module_type_typeof s
+
+    (** analyse of a Parsetree.module_type and a Types.module_type.*)
     and analyse_module_kind env current_module_name module_type sig_module_type =
       match module_type.Parsetree.pmty_desc with
         Parsetree.Pmty_ident longident ->
-	  let k = analyse_module_type_kind env current_module_name module_type sig_module_type in
+          let k = analyse_module_type_kind env current_module_name module_type sig_module_type in
           Module_with ( k, "" )
 
       | Parsetree.Pmty_signature signature ->
@@ -1124,26 +1108,26 @@ module Analyser =
           (
            match sig_module_type with
              Types.Tmty_functor (ident, param_module_type, body_module_type) ->
-	       let loc_start = pmodule_type2.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
+               let loc_start = pmodule_type2.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
                let loc_end = pmodule_type2.Parsetree.pmty_loc.Location.loc_end.Lexing.pos_cnum in
-	       let mp_type_code = get_string_of_file loc_start loc_end in
-	       print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
-	       let mp_kind = analyse_module_type_kind env
-		   current_module_name pmodule_type2 param_module_type
-	       in
+               let mp_type_code = get_string_of_file loc_start loc_end in
+               print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
+               let mp_kind = analyse_module_type_kind env
+                   current_module_name pmodule_type2 param_module_type
+               in
                let param =
                  {
                    mp_name = Name.from_ident ident ;
                    mp_type = Odoc_env.subst_module_type env param_module_type ;
-		   mp_type_code = mp_type_code ;
-		   mp_kind = mp_kind ;
+                   mp_type_code = mp_type_code ;
+                   mp_kind = mp_kind ;
                  }
                in
                let k = analyse_module_kind env
-		   current_module_name
-		   module_type2
-		   body_module_type
-	       in
+                   current_module_name
+                   module_type2
+                   body_module_type
+               in
                Module_functor (param, k)
 
            | _ ->
@@ -1159,6 +1143,11 @@ module Analyser =
            let k = analyse_module_type_kind env current_module_name module_type2 sig_module_type in
            Module_with (k, s)
           )
+      | Parsetree.Pmty_typeof module_expr ->
+          let loc_start = module_expr.Parsetree.pmod_loc.Location.loc_start.Lexing.pos_cnum in
+          let loc_end = module_expr.Parsetree.pmod_loc.Location.loc_end.Lexing.pos_cnum in
+          let s = get_string_of_file loc_start loc_end in
+          Module_typeof s
 
     (** Analyse of a Parsetree.class_type and a Types.class_type to return a couple
        (class parameters, class_kind).*)
@@ -1180,11 +1169,6 @@ module Analyser =
            ([], k)
 
       | (Parsetree.Pcty_signature (_, class_type_field_list), Types.Tcty_signature class_signature) ->
-          print_DEBUG "Types.Tcty_signature class_signature";
-          let f_DEBUG var (mutable_flag, type_exp) = print_DEBUG var in
-          Types.Vars.iter f_DEBUG class_signature.Types.cty_vars;
-          print_DEBUG ("Type de la classe "^current_class_name^" : ");
-          print_DEBUG (Odoc_print.string_of_type_expr class_signature.Types.cty_self);
           (* we get the elements of the class in class_type_field_list *)
           let (inher_l, ele) = analyse_class_elements env current_class_name
               last_pos
@@ -1234,11 +1218,6 @@ module Analyser =
            k
 
       | (Parsetree.Pcty_signature (_, class_type_field_list), Types.Tcty_signature class_signature) ->
-          print_DEBUG "Types.Tcty_signature class_signature";
-          let f_DEBUG var (mutable_flag, type_exp) = print_DEBUG var in
-          Types.Vars.iter f_DEBUG class_signature.Types.cty_vars;
-          print_DEBUG ("Type de la classe "^current_class_name^" : ");
-          print_DEBUG (Odoc_print.string_of_type_expr class_signature.Types.cty_self);
           (* we get the elements of the class in class_type_field_list *)
           let (inher_l, ele) = analyse_class_elements env current_class_name
               last_pos
@@ -1279,7 +1258,7 @@ module Analyser =
           raise (Failure "analyse_class_type_kind pas de correspondance dans le match")
 
     let analyse_signature source_file input_file
-	(ast : Parsetree.signature) (signat : Types.signature) =
+        (ast : Parsetree.signature) (signat : Types.signature) =
       let complete_source_file =
         try
           let curdir = Sys.getcwd () in
@@ -1301,13 +1280,13 @@ module Analyser =
       in
       let (len,info_opt) = My_ir.first_special !file_name !file in
       let elements =
-	analyse_parsetree Odoc_env.empty signat mod_name len (String.length !file) ast
+        analyse_parsetree Odoc_env.empty signat mod_name len (String.length !file) ast
       in
       let code_intf =
-	if !Odoc_args.keep_code then
-	  Some !file
-	else
-	  None
+        if !Odoc_args.keep_code then
+          Some !file
+        else
+          None
       in
       {
         m_name = mod_name ;
@@ -1318,9 +1297,9 @@ module Analyser =
         m_kind = Module_struct elements ;
         m_loc = { loc_impl = None ; loc_inter = Some (!file_name, 0) } ;
         m_top_deps = [] ;
-	m_code = None ;
-	m_code_intf = code_intf ;
-	m_text_only = false ;
+        m_code = None ;
+        m_code_intf = code_intf ;
+        m_text_only = false ;
       }
 
     end
