@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: natdynlink.ml 8993 2008-08-28 22:17:51Z frisch $ *)
+(* $Id$ *)
 
 (* Dynamic loading of .cmx files *)
 
@@ -40,22 +40,14 @@ type error =
 
 exception Error of error
 
-(* Copied from other places to avoid dependencies *)
+open Cmx_format
 
-type dynunit = {
-  name: string;
-  crc: Digest.t;
-  imports_cmi: (string * Digest.t) list;
-  imports_cmx: (string * Digest.t) list;
-  defines: string list;
-}
+(* Copied from config.ml to avoid dependencies *)
+let cmxs_magic_number = "Caml2007D001"
 
-type dynheader = {
-  magic: string;
-  units: dynunit list;
-}
-
-let dyn_magic_number = "Caml2007D001"
+(* Copied from compilenv.ml to avoid dependencies *)
+let cmx_not_found_crc =
+  "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000"
 
 let dll_filename fname =
   if Filename.is_implicit fname then Filename.concat (Sys.getcwd ()) fname
@@ -70,12 +62,10 @@ let read_file filename priv =
   then raise (Error (Cannot_open_dll (Obj.magic res)));
 
   let header : dynheader = Marshal.from_string data 0 in
-  if header.magic <> dyn_magic_number
+  if header.dynu_magic <> cmxs_magic_number
   then raise(Error(Not_a_bytecode_file dll));
-  (dll, handle, header.units)
+  (dll, handle, header.dynu_units)
 
-let cmx_not_found_crc =
-  "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000"
 
 
 (* Management of interface and implementation CRCs *)
@@ -112,8 +102,8 @@ let default_available_units () =
       (fun st (name,crc_intf,crc_impl,syms) ->
         rank := !rank + List.length syms;
         {
-	 ifaces = StrMap.add name (crc_intf,exe) st.ifaces;
-	 implems = StrMap.add name (crc_impl,exe,Check_inited !rank) st.implems;
+         ifaces = StrMap.add name (crc_intf,exe) st.ifaces;
+         implems = StrMap.add name (crc_impl,exe,Check_inited !rank) st.implems;
         }
       )
       empty_state
@@ -127,48 +117,48 @@ let init () =
 let add_check_ifaces allow_ext filename ui ifaces =
   List.fold_left
     (fun ifaces (name, crc) ->
-       if name = ui.name
+       if name = ui.dynu_name
        then StrMap.add name (crc,filename) ifaces
        else
-	 try
-	   let (old_crc,old_src) = StrMap.find name ifaces in
-	   if old_crc <> crc
-	   then raise(Error(Inconsistent_import(name)))
-	   else ifaces
-	 with Not_found ->
-	   if allow_ext then StrMap.add name (crc,filename) ifaces
-	   else raise (Error(Unavailable_unit name))
-    ) ifaces ui.imports_cmi
+         try
+           let (old_crc,old_src) = StrMap.find name ifaces in
+           if old_crc <> crc
+           then raise(Error(Inconsistent_import(name)))
+           else ifaces
+         with Not_found ->
+           if allow_ext then StrMap.add name (crc,filename) ifaces
+           else raise (Error(Unavailable_unit name))
+    ) ifaces ui.dynu_imports_cmi
 
 let check_implems filename ui implems =
   List.iter
     (fun (name, crc) ->
        match name with
-	 |"Out_of_memory"
-	 |"Sys_error"
-	 |"Failure"
-	 |"Invalid_argument"
-	 |"End_of_file"
-	 |"Division_by_zero"
-	 |"Not_found"
-	 |"Match_failure"
-	 |"Stack_overflow"
-	 |"Sys_blocked_io"
-	 |"Assert_failure"
-	 |"Undefined_recursive_module" -> ()
-	 | _ ->
+         |"Out_of_memory"
+         |"Sys_error"
+         |"Failure"
+         |"Invalid_argument"
+         |"End_of_file"
+         |"Division_by_zero"
+         |"Not_found"
+         |"Match_failure"
+         |"Stack_overflow"
+         |"Sys_blocked_io"
+         |"Assert_failure"
+         |"Undefined_recursive_module" -> ()
+         | _ ->
        try
-	 let (old_crc,old_src,state) = StrMap.find name implems in
-	 if crc <> cmx_not_found_crc && old_crc <> crc
-	 then raise(Error(Inconsistent_implementation(name)))
-	 else match state with
-	   | Check_inited i ->
-	       if ndl_globals_inited() < i
-	       then raise(Error(Unavailable_unit name))
-	   | Loaded -> ()
+         let (old_crc,old_src,state) = StrMap.find name implems in
+         if crc <> cmx_not_found_crc && old_crc <> crc
+         then raise(Error(Inconsistent_implementation(name)))
+         else match state with
+           | Check_inited i ->
+               if ndl_globals_inited() < i
+               then raise(Error(Unavailable_unit name))
+           | Loaded -> ()
        with Not_found ->
-	 raise (Error(Unavailable_unit name))
-    ) ui.imports_cmx
+         raise (Error(Unavailable_unit name))
+    ) ui.dynu_imports_cmx
 
 let loadunits filename handle units state =
   let new_ifaces =
@@ -178,11 +168,11 @@ let loadunits filename handle units state =
   let new_implems =
     List.fold_left
       (fun accu ui ->
-	 check_implems filename ui accu;
-	 StrMap.add ui.name (ui.crc,filename,Loaded) accu)
+         check_implems filename ui accu;
+         StrMap.add ui.dynu_name (ui.dynu_crc,filename,Loaded) accu)
       state.implems units in
 
-  let defines = List.flatten (List.map (fun ui -> ui.defines) units) in
+  let defines = List.flatten (List.map (fun ui -> ui.dynu_defines) units) in
 
   ndl_run handle "_shared_startup";
   List.iter (ndl_run handle) defines;
@@ -203,8 +193,8 @@ let allow_only names =
   let ifaces =
     List.fold_left
       (fun ifaces name ->
-	 try StrMap.add name (StrMap.find name old) ifaces
-	 with Not_found -> ifaces)
+         try StrMap.add name (StrMap.find name old) ifaces
+         with Not_found -> ifaces)
       StrMap.empty names in
   global_state := { !global_state with ifaces = ifaces };
   allow_extension := false
