@@ -7,11 +7,12 @@
 (*   Copyright 2000 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: dummyUnix.mli,v 1.1 2000/02/01 05:43:24 garrigue Exp $ *)
+(* $Id$ *)
 
 module Mutex : sig
   type t
@@ -22,11 +23,5 @@ end
 
 module Thread : sig
   type t
-  external create : ('a -> 'b) -> 'a -> t = "caml_input"
-end
-
-module ThreadUnix : sig
-  open Unix
-  external read : file_descr -> buf:string -> pos:int -> len:int -> int
-      = "caml_input"
+  external create : ('a -> 'b) -> 'a -> t = "caml_ml_input"
 end
