@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: jg_text.ml,v 1.10 2001/12/07 13:39:59 xleroy Exp $ *)
+(* $Id$ *)
 
 open StdLabels
 open Tk
@@ -48,7 +48,7 @@ let goto_tag tw ~tag =
 
 let search_string tw =
   let tl = Jg_toplevel.titled "Search" in
-  Wm.transient_set tl ~master:Widget.default_toplevel;
+  Wm.transient_set tl ~master:(Winfo.toplevel tw);
   let fi = Frame.create tl
   and fd = Frame.create tl
   and fm = Frame.create tl
