@@ -7,16 +7,17 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: jg_tk.ml,v 1.5 2000/08/25 08:20:49 garrigue Exp $ *)
+(* $Id$ *)
 
 open Tk
 
-let tpos x : textIndex = `Linechar (1,0), [`Char x]
-and tposend x : textIndex = `End, [`Char (-x)]
+let tpos ?(modi=[]) x : textIndex = `Linechar (1,0), `Char x :: modi
+and tposend ?(modi=[]) x : textIndex = `End, `Char (-x) :: modi
 let tstart : textIndex = `Linechar (1,0), []
 and tend : textIndex = `End, []
 
