@@ -1,20 +1,20 @@
-/*************************************************************************/
-/*                                                                       */
-/*                Objective Caml LablTk library                          */
-/*                                                                       */
-/*         Francois Rouaix, Francois Pessaux and Jun Furuse              */
-/*               projet Cristal, INRIA Rocquencourt                      */
-/*            Jacques Garrigue, Kyoto University RIMS                    */
-/*                                                                       */
-/*   Copyright 1999 Institut National de Recherche en Informatique et    */
-/*   en Automatique and Kyoto University.  All rights reserved.          */
-/*   This file is distributed under the terms of the GNU Library         */
-/*   General Public License, with the special exception on linking       */
-/*   described in file ../../../LICENSE.                                 */
-/*                                                                       */
-/*************************************************************************/
+/***********************************************************************/
+/*                                                                     */
+/*                 MLTk, Tcl/Tk interface of Objective Caml            */
+/*                                                                     */
+/*    Francois Rouaix, Francois Pessaux, Jun Furuse and Pierre Weis    */
+/*               projet Cristal, INRIA Rocquencourt                    */
+/*            Jacques Garrigue, Kyoto University RIMS                  */
+/*                                                                     */
+/*  Copyright 2002 Institut National de Recherche en Informatique et   */
+/*  en Automatique and Kyoto University.  All rights reserved.         */
+/*  This file is distributed under the terms of the GNU Library        */
+/*  General Public License, with the special exception on linking      */
+/*  described in file LICENSE found in the Objective Caml source tree. */
+/*                                                                     */
+/***********************************************************************/
 
-/* $Id: cltkCaml.c,v 1.7 2001/12/07 13:40:08 xleroy Exp $ */
+/* $Id$ */
 
 #include <tcl.h>
 #include <tk.h>
@@ -28,7 +28,8 @@ value * tkerror_exn = NULL;
 value * handler_code = NULL;
 
 /* The Tcl command for evaluating callback in Caml */
-int CamlCBCmd(ClientData clientdata, Tcl_Interp *interp, int argc, char **argv)
+int CamlCBCmd(ClientData clientdata, Tcl_Interp *interp,
+              int argc, CONST84 char **argv)
 {
   CheckInit();
 
@@ -38,7 +39,8 @@ int CamlCBCmd(ClientData clientdata, Tcl_Interp *interp, int argc, char **argv)
     int id;
     if (Tcl_GetInt(interp, argv[1], &id) != TCL_OK)
       return TCL_ERROR;
-    callback2(*handler_code,Val_int(id),copy_string_list(argc - 2,&argv[2]));
+    callback2(*handler_code,Val_int(id),
+              copy_string_list(argc - 2,(char **)&argv[2]));
     /* Never fails (Caml would have raised an exception) */
     /* but result may have been set by callback */
     return TCL_OK;
@@ -61,7 +63,7 @@ CAMLprim value camltk_return (value v)
 }
 
 /* Note: raise_with_string WILL copy the error message */
-void tk_error(char *errmsg)
+CAMLprim void tk_error(char *errmsg)
 {
   raise_with_string(*tkerror_exn, errmsg);
 }
