@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: envir.c,v 1.10 2005/03/24 17:20:53 doligez Exp $ */
+/* $Id$ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -22,5 +22,9 @@ extern char ** environ;
 
 CAMLprim value unix_environment(value unit)
 {
-  return copy_string_array((const char**)environ);
+  if (environ != NULL) {
+    return copy_string_array((const char**)environ);
+  } else {
+    return Atom(0);
+  }
 }
