@@ -6,20 +6,25 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: fork.c,v 1.6 1999/11/17 18:58:00 xleroy Exp $ */
+/* $Id$ */
 
 #include <mlvalues.h>
+#include <debugger.h>
 #include "unixsupport.h"
 
-value unix_fork(value unit)               /* ML */
+CAMLprim value unix_fork(value unit)
 {
   int ret;
   ret = fork();
   if (ret == -1) uerror("fork", Nothing);
+  if (caml_debugger_in_use)
+    if ((caml_debugger_fork_mode && ret == 0) ||
+        (!caml_debugger_fork_mode && ret != 0))
+      caml_debugger_cleanup_fork();
   return Val_int(ret);
 }
-
