@@ -11,13 +11,13 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: socket.c 4765 2002-04-30 15:00:48Z xleroy $ */
+/* $Id$ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 int socket_domain_table[] = {
-  PF_UNIX, PF_INET
+  PF_UNIX, PF_INET /*, PF_INET6 */
 };
 
 int socket_type_table[] = {
@@ -30,13 +30,19 @@ CAMLprim value unix_socket(domain, type, proto)
   SOCKET s;
   int oldvalue, oldvaluelen, newvalue, retcode;
 
+  /* IPv6 requires WinSock2, we must raise an error on PF_INET6 */
+  if (Int_val(domain) >= sizeof(socket_domain_table)/sizeof(int)) {
+    win32_maperr(WSAEPFNOSUPPORT);
+    uerror("socket", Nothing);
+  }
+
   oldvaluelen = sizeof(oldvalue);
   retcode = getsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
                        (char *) &oldvalue, &oldvaluelen);
   if (retcode == 0) {
     /* Set sockets to synchronous mode */
     newvalue = SO_SYNCHRONOUS_NONALERT;
-    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, 
+    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
                (char *) &newvalue, sizeof(newvalue));
   }
   s = socket(socket_domain_table[Int_val(domain)],
@@ -44,7 +50,7 @@ CAMLprim value unix_socket(domain, type, proto)
                    Int_val(proto));
   if (retcode == 0) {
     /* Restore initial mode */
-    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, 
+    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
                (char *) &oldvalue, oldvaluelen);
   }
   if (s == INVALID_SOCKET) {
