@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asttypes.mli,v 1.8 1999/12/06 17:05:19 garrigue Exp $ *)
+(* $Id$ *)
 
 (* Auxiliary a.s.t. types used by parsetree and typedtree. *)
 
@@ -19,6 +19,9 @@ type constant =
   | Const_char of char
   | Const_string of string
   | Const_float of string
+  | Const_int32 of int32
+  | Const_int64 of int64
+  | Const_nativeint of nativeint
 
 type rec_flag = Nonrecursive | Recursive | Default
 
@@ -30,4 +33,8 @@ type mutable_flag = Immutable | Mutable
 
 type virtual_flag = Virtual | Concrete
 
+type override_flag = Override | Fresh
+
+type closed_flag = Closed | Open
+
 type label = string
