@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parsetree.mli,v 1.43 2006/04/05 02:28:13 garrigue Exp $ *)
+(* $Id$ *)
 
 (* Abstract syntax tree produced by parsing *)
 
@@ -22,7 +22,7 @@ type core_type =
   { ptyp_desc: core_type_desc;
     ptyp_loc: Location.t }
 
-and core_type_desc = 
+and core_type_desc =
     Ptyp_any
   | Ptyp_var of string
   | Ptyp_arrow of label * core_type * core_type
@@ -33,6 +33,9 @@ and core_type_desc =
   | Ptyp_alias of core_type * string
   | Ptyp_variant of row_field list * bool * label list option
   | Ptyp_poly of string list * core_type
+  | Ptyp_package of package_type
+
+and package_type = Longident.t * (string * core_type) list
 
 and core_field_type =
   { pfield_desc: core_field_desc;
@@ -46,7 +49,7 @@ and row_field =
     Rtag of label * bool * core_type list
   | Rinherit of core_type
 
-(* XXX Type expressions for the class language *)
+(* Type expressions for the class language *)
 
 type 'a class_infos =
   { pci_virt: virtual_flag;
@@ -70,11 +73,12 @@ and pattern_desc =
   | Ppat_tuple of pattern list
   | Ppat_construct of Longident.t * pattern option * bool
   | Ppat_variant of label * pattern option
-  | Ppat_record of (Longident.t * pattern) list
+  | Ppat_record of (Longident.t * pattern) list * closed_flag
   | Ppat_array of pattern list
   | Ppat_or of pattern * pattern
   | Ppat_constraint of pattern * core_type
   | Ppat_type of Longident.t
+  | Ppat_lazy of pattern
 
 type expression =
   { pexp_desc: expression_desc;
@@ -111,6 +115,9 @@ and expression_desc =
   | Pexp_lazy of expression
   | Pexp_poly of expression * core_type option
   | Pexp_object of class_structure
+  | Pexp_newtype of string * expression
+  | Pexp_pack of module_expr * package_type
+  | Pexp_open of Longident.t * expression
 
 (* Value descriptions *)
 
@@ -124,16 +131,16 @@ and type_declaration =
   { ptype_params: string list;
     ptype_cstrs: (core_type * core_type * Location.t) list;
     ptype_kind: type_kind;
+    ptype_private: private_flag;
     ptype_manifest: core_type option;
     ptype_variance: (bool * bool) list;
     ptype_loc: Location.t }
 
 and type_kind =
     Ptype_abstract
-  | Ptype_variant of (string * core_type list * Location.t) list * private_flag
+  | Ptype_variant of (string * core_type list * Location.t) list
   | Ptype_record of
-      (string * mutable_flag * core_type * Location.t) list * private_flag
-  | Ptype_private
+      (string * mutable_flag * core_type * Location.t) list
 
 and exception_declaration = core_type list
 
@@ -178,11 +185,11 @@ and class_expr_desc =
 and class_structure = pattern * class_field list
 
 and class_field =
-    Pcf_inher of class_expr * string option
+    Pcf_inher of override_flag * class_expr * string option
   | Pcf_valvirt of (string * mutable_flag * core_type * Location.t)
-  | Pcf_val   of (string * mutable_flag * expression * Location.t)
+  | Pcf_val of (string * mutable_flag * override_flag * expression * Location.t)
   | Pcf_virt  of (string * private_flag * core_type * Location.t)
-  | Pcf_meth  of (string * private_flag * expression * Location.t)
+  | Pcf_meth of (string * private_flag *override_flag * expression * Location.t)
   | Pcf_cstr  of (core_type * core_type * Location.t)
   | Pcf_let   of rec_flag * (pattern * expression) list * Location.t
   | Pcf_init  of expression
@@ -200,6 +207,7 @@ and module_type_desc =
   | Pmty_signature of signature
   | Pmty_functor of string * module_type * module_type
   | Pmty_with of module_type * (Longident.t * with_constraint) list
+  | Pmty_typeof of module_expr
 
 and signature = signature_item list
 
@@ -226,8 +234,10 @@ and modtype_declaration =
 and with_constraint =
     Pwith_type of type_declaration
   | Pwith_module of Longident.t
+  | Pwith_typesubst of type_declaration
+  | Pwith_modsubst of Longident.t
 
-(* Value expressions for the module language *)
+(* value expressions for the module language *)
 
 and module_expr =
   { pmod_desc: module_expr_desc;
@@ -239,6 +249,7 @@ and module_expr_desc =
   | Pmod_functor of string * module_type * module_expr
   | Pmod_apply of module_expr * module_expr
   | Pmod_constraint of module_expr * module_type
+  | Pmod_unpack of expression * package_type
 
 and structure = structure_item list
 
