@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: array.ml,v 1.23 2003/12/31 14:20:39 doligez Exp $ *)
+(* $Id$ *)
 
 (* Array operations *)
 
@@ -29,7 +29,7 @@ let init l f =
    for i = 1 to pred l do
      unsafe_set res i (f i)
    done;
-   res 
+   res
 
 let make_matrix sx sy init =
   let res = create sx [||] in
@@ -54,8 +54,8 @@ let append a1 a2 =
   let l1 = length a1 and l2 = length a2 in
   if l1 = 0 && l2 = 0 then [||] else begin
     let r = create (l1 + l2) (unsafe_get (if l1 > 0 then a1 else a2) 0) in
-    for i = 0 to l1 - 1 do unsafe_set r i (unsafe_get a1 i) done;  
-    for i = 0 to l2 - 1 do unsafe_set r (i + l1) (unsafe_get a2 i) done;  
+    for i = 0 to l1 - 1 do unsafe_set r i (unsafe_get a1 i) done;
+    for i = 0 to l2 - 1 do unsafe_set r (i + l1) (unsafe_get a2 i) done;
     r
   end
 
@@ -67,7 +67,7 @@ let concat_aux init al =
   let res = create (size 0 al) init in
   let rec fill pos = function
     | [] -> ()
-    | h::t -> 
+    | h::t ->
         for i = 0 to length h - 1 do
           unsafe_set res (pos + i) (unsafe_get h i);
         done;
@@ -78,10 +78,11 @@ let concat_aux init al =
 ;;
 
 let concat al =
-  let rec find_init = function
-      [] -> [||]
+  let rec find_init aa =
+    match aa with
+    | [] -> [||]
     | a :: rem ->
-        if length a > 0 then concat_aux (unsafe_get a 0) al else find_init rem
+        if length a > 0 then concat_aux (unsafe_get a 0) aa else find_init rem
   in find_init al
 
 let sub a ofs len =
