@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: buffer.ml,v 1.18 2005/10/25 18:34:07 doligez Exp $ *)
+(* $Id$ *)
 
 (* Extensible buffers *)
 
@@ -39,6 +39,14 @@ let sub b ofs len =
   end
 ;;
 
+let blit src srcoff dst dstoff len =
+  if len < 0 || srcoff < 0 || srcoff > src.position - len
+             || dstoff < 0 || dstoff > (String.length dst) - len
+  then invalid_arg "Buffer.blit"
+  else
+    String.blit src.buffer srcoff dst dstoff len
+;;
+
 let nth b ofs =
   if ofs < 0 || ofs >= b.position then
    invalid_arg "Buffer.nth"
@@ -92,6 +100,8 @@ let add_buffer b bs =
   add_substring b bs.buffer 0 bs.position
 
 let add_channel b ic len =
+  if len < 0 || len > Sys.max_string_length then   (* PR#5004 *)
+    invalid_arg "Buffer.add_channel";
   if b.position + len > b.length then resize b len;
   really_input ic b.buffer b.position len;
   b.position <- b.position + len
@@ -126,12 +136,13 @@ let advance_to_non_alpha s start =
       ''|''|''|''|''|''|''|
       ''|''|''|''|''|''|''|
       ''|''|''|''|''|''|'' ->
-        advance (i + 1) lim
+      advance (i + 1) lim
     | _ -> i in
   advance start (String.length s);;
 
 (* We are just at the beginning of an ident in s, starting at start. *)
-let find_ident s start =
+let find_ident s start lim =
+  if start >= lim then raise Not_found else
   match s.[start] with
   (* Parenthesized ident ? *)
   | '(' | '{' as c ->
@@ -152,19 +163,21 @@ let add_substitute b f s =
       match s.[i] with
       | '$' as current when previous = '\\' ->
          add_char b current;
-         subst current (i + 1)
+         subst ' ' (i + 1)
       | '$' ->
-         let ident, next_i = find_ident s (i + 1) in
+         let j = i + 1 in
+         let ident, next_i = find_ident s j lim in
          add_string b (f ident);
          subst ' ' next_i
       | current when previous == '\\' ->
          add_char b '\\';
          add_char b current;
-         subst current (i + 1)
+         subst ' ' (i + 1)
       | '\\' as current ->
          subst current (i + 1)
       | current ->
          add_char b current;
          subst current (i + 1)
-    end in
+    end else
+    if previous = '\\' then add_char b previous in
   subst ' ' 0;;
