@@ -11,15 +11,15 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printexc.mli 9335 2009-09-16 13:34:57Z xclerc $ *)
+(* $Id$ *)
 
 (** Facilities for printing exceptions. *)
 
-val to_string : exn -> string
+val to_string: exn -> string
 (** [Printexc.to_string e] returns a string representation of
    the exception [e]. *)
 
-val print : ('a -> 'b) -> 'a -> 'b
+val print: ('a -> 'b) -> 'a -> 'b
 (** [Printexc.print fn x] applies [fn] to [x] and returns the result.
    If the evaluation of [fn x] raises any exception, the
    name of the exception is printed on standard error output,
@@ -27,7 +27,7 @@ val print : ('a -> 'b) -> 'a -> 'b
    The typical use is to catch and report exceptions that
    escape a function application. *)
 
-val catch : ('a -> 'b) -> 'a -> 'b
+val catch: ('a -> 'b) -> 'a -> 'b
 (** [Printexc.catch fn x] is similar to {!Printexc.print}, but
    aborts the program with exit code 2 after printing the
    uncaught exception.  This function is deprecated: the runtime
@@ -41,29 +41,41 @@ val print_backtrace: out_channel -> unit
 (** [Printexc.print_backtrace oc] prints an exception backtrace
     on the output channel [oc].  The backtrace lists the program
     locations where the most-recently raised exception was raised
-    and where it was propagated through function calls. *)
+    and where it was propagated through function calls.
+    @since 3.11.0
+*)
 
 val get_backtrace: unit -> string
 (** [Printexc.get_backtrace ()] returns a string containing the
     same exception backtrace that [Printexc.print_backtrace] would
-    print. *)
+    print.
+    @since 3.11.0
+*)
 
 val record_backtrace: bool -> unit
 (** [Printexc.record_backtrace b] turns recording of exception backtraces
     on (if [b = true]) or off (if [b = false]).  Initially, backtraces
     are not recorded, unless the [b] flag is given to the program
-    through the [OCAMLRUNPARAM] variable.  *)
+    through the [OCAMLRUNPARAM] variable.
+    @since 3.11.0
+*)
 
 val backtrace_status: unit -> bool
 (** [Printexc.backtrace_status()] returns [true] if exception
-    backtraces are currently recorded, [false] if not. *)
+    backtraces are currently recorded, [false] if not.
+    @since 3.11.0
+*)
+
+val register_printer: (exn -> string option) -> unit
+(** [Printexc.register_printer fn] registers [fn] as an exception
+    printer.  The printer should return [None] or raise an exception
+    if it does not know how to convert the passed exception, and [Some
+    s] with [s] the resulting string if it can convert the passed
+    exception. Exceptions raised by the printer are ignored.
 
-val register_printer : (exn -> string option) -> unit
-(** [Printexc.register_printer fn] registers [fn] as an exception printer.
-    The printer should return [None] if it does not know how to convert
-    the passed exception, and [Some s] with [s] the resulting string if
-    it can convert the passed exception.
     When converting an exception into a string, the printers will be invoked
     in the reverse order of their registrations, until a printer returns
     a [Some s] value (if no such printer exists, the runtime will use a
-    generic printer). *)
+    generic printer).
+    @since 3.11.2
+*)
