@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: sys.mli 7927 2007-02-26 14:21:57Z xleroy $ *)
+(* $Id$ *)
 
 (** System interface. *)
 
@@ -30,7 +30,9 @@ external file_exists : string -> bool = "caml_sys_file_exists"
 external is_directory : string -> bool = "caml_sys_is_directory"
 (** Returns [true] if the given name refers to a directory,
     [false] if it refers to another kind of file.
-    Raise [Sys_error] if no file exists with the given name. *)
+    Raise [Sys_error] if no file exists with the given name.
+    @since 3.10.0
+*)
 
 external remove : string -> unit = "caml_sys_remove"
 (** Remove the given file name from the file system. *)
@@ -82,6 +84,9 @@ val word_size : int
 (** Size of one word on the machine currently executing the Caml
    program, in bits: 32 or 64. *)
 
+val big_endian : bool
+(** Whether the machine currently executing the Caml program is big-endian. *)
+
 val max_string_length : int
 (** Maximum length of a string. *)
 
