@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: toploop.ml,v 1.95 2007/12/04 13:38:58 doligez Exp $ *)
+(* $Id$ *)
 
 (* The interactive toplevel loop *)
 
@@ -322,12 +322,12 @@ let first_line = ref true
 let got_eof = ref false;;
 
 let read_input_default prompt buffer len =
-  output_string stdout prompt; flush stdout;
+  output_string Pervasives.stdout prompt; flush Pervasives.stdout;
   let i = ref 0 in
   try
     while true do
       if !i >= len then raise Exit;
-      let c = input_char stdin in
+      let c = input_char Pervasives.stdin in
       buffer.[!i] <- c;
       incr i;
       if c = '\n' then raise Exit;
@@ -403,7 +403,8 @@ let loop ppf =
   fprintf ppf "        Objective Caml version %s@.@." Config.version;
   initialize_toplevel_env ();
   let lb = Lexing.from_function refill_lexbuf in
-  Location.input_name := "";
+  Location.init lb "//toplevel//";
+  Location.input_name := "//toplevel//";
   Location.input_lexbuf := Some lb;
   Sys.catch_break true;
   load_ocamlinit ppf;
