@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: includecore.mli 2553 1999-11-17 18:59:06Z xleroy $ *)
+(* $Id$ *)
 
 (* Inclusion checks for the core language *)
 
@@ -19,13 +19,31 @@ open Typedtree
 
 exception Dont_match
 
+type type_mismatch =
+    Arity
+  | Privacy
+  | Kind
+  | Constraint
+  | Manifest
+  | Variance
+  | Field_type of string
+  | Field_mutable of string
+  | Field_arity of string
+  | Field_names of int * string * string
+  | Field_missing of bool * string
+  | Record_representation of bool
+
 val value_descriptions:
-        Env.t -> value_description -> value_description -> module_coercion
+    Env.t -> value_description -> value_description -> module_coercion
 val type_declarations:
-        Env.t -> Ident.t -> type_declaration -> type_declaration -> bool
+    Env.t -> Ident.t ->
+    type_declaration -> type_declaration -> type_mismatch list
 val exception_declarations:
-        Env.t -> exception_declaration -> exception_declaration -> bool
+    Env.t -> exception_declaration -> exception_declaration -> bool
 (*
 val class_types:
         Env.t -> class_type -> class_type -> bool
 *)
+
+val report_type_mismatch:
+    string -> string -> string -> Format.formatter -> type_mismatch list -> unit
