@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedecl.mli,v 1.31.4.1 2009/06/08 04:08:14 garrigue Exp $ *)
+(* $Id$ *)
 
 (* Typing of type definitions and primitive definitions *)
 
@@ -30,7 +30,7 @@ val transl_value_decl:
     Env.t -> Parsetree.value_description -> value_description
 
 val transl_with_constraint:
-    Env.t -> Ident.t -> Path.t option ->
+    Env.t -> Ident.t -> Path.t option -> type_declaration ->
     Parsetree.type_declaration -> type_declaration
 
 val abstract_type_decl: int -> type_declaration
@@ -50,14 +50,14 @@ val compute_variance_decls:
        cltype_declaration * ((bool * bool) list * Location.t)) list ->
     (type_declaration * type_declaration * class_declaration *
        cltype_declaration) list
-    
+
 type error =
     Repeated_parameter
   | Duplicate_constructor of string
   | Too_many_constructors
   | Duplicate_label of string
   | Recursive_abbrev of string
-  | Definition_mismatch of type_expr
+  | Definition_mismatch of type_expr * Includecore.type_mismatch list
   | Constraint_failed of type_expr * type_expr
   | Unconsistent_constraint of (type_expr * type_expr) list
   | Type_clash of (type_expr * type_expr) list
