@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedtree.ml,v 1.37.8.2 2007/07/10 07:34:35 garrigue Exp $ *)
+(* $Id$ *)
 
 (* Abstract syntax tree after typing *)
 
@@ -37,6 +37,7 @@ and pattern_desc =
   | Tpat_record of (label_description * pattern) list
   | Tpat_array of pattern list
   | Tpat_or of pattern * pattern * row_desc option
+  | Tpat_lazy of pattern
 
 type partial = Partial | Total
 type optional = Required | Optional
@@ -78,6 +79,7 @@ and expression_desc =
   | Texp_assertfalse
   | Texp_lazy of expression
   | Texp_object of class_structure * class_signature * string list
+  | Texp_pack of module_expr
 
 and meth =
     Tmeth_name of string
@@ -126,6 +128,7 @@ and module_expr_desc =
   | Tmod_functor of Ident.t * module_type * module_expr
   | Tmod_apply of module_expr * module_expr * module_coercion
   | Tmod_constraint of module_expr * module_type * module_coercion
+  | Tmod_unpack of expression * module_type
 
 and structure = structure_item list
 
@@ -162,6 +165,7 @@ let iter_pattern_desc f = function
       List.iter (fun (lbl, pat) -> f pat) lbl_pat_list
   | Tpat_array patl -> List.iter f patl
   | Tpat_or(p1, p2, _) -> f p1; f p2
+  | Tpat_lazy p -> f p
   | Tpat_any
   | Tpat_var _
   | Tpat_constant _ -> ()
@@ -178,6 +182,7 @@ let map_pattern_desc f d =
       Tpat_construct (c, List.map f pats)
   | Tpat_array pats ->
       Tpat_array (List.map f pats)
+  | Tpat_lazy p1 -> Tpat_lazy (f p1)
   | Tpat_variant (x1, Some p1, x2) ->
       Tpat_variant (x1, Some (f p1), x2)
   | Tpat_or (p1,p2,path) ->
