@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typemod.ml 10532 2010-06-07 08:24:02Z garrigue $ *)
+(* $Id$ *)
 
 (* Type-checking of the module language *)
 
@@ -91,6 +91,16 @@ let rec make_params n = function
 
 let wrap_param s = {ptyp_desc=Ptyp_var s; ptyp_loc=Location.none}
 
+let make_next_first rs rem =
+  if rs = Trec_first then
+    match rem with
+      Tsig_type (id, decl, Trec_next) :: rem ->
+        Tsig_type (id, decl, Trec_first) :: rem
+    | Tsig_module (id, mty, Trec_next) :: rem ->
+        Tsig_module (id, mty, Trec_first) :: rem
+    | _ -> rem
+  else rem
+
 let merge_constraint initial_env loc sg lid constr =
   let real_id = ref None in
   let rec merge env sg namelist row_id =
@@ -113,7 +123,7 @@ let merge_constraint initial_env loc sg lid constr =
         and id_row = Ident.create (s^"#row") in
         let initial_env = Env.add_type id_row decl_row initial_env in
         let newdecl = Typedecl.transl_with_constraint
-                        initial_env id (Some(Pident id_row)) sdecl in
+                        initial_env id (Some(Pident id_row)) decl sdecl in
         check_type_decl env id row_id newdecl decl rs rem;
         let decl_row = {decl_row with type_params = newdecl.type_params} in
         let rs' = if rs = Trec_first then Trec_not else rs in
@@ -121,7 +131,7 @@ let merge_constraint initial_env loc sg lid constr =
     | (Tsig_type(id, decl, rs) :: rem, [s], Pwith_type sdecl)
       when Ident.name id = s ->
         let newdecl =
-          Typedecl.transl_with_constraint initial_env id None sdecl in
+          Typedecl.transl_with_constraint initial_env id None decl sdecl in
         check_type_decl env id row_id newdecl decl rs rem;
         Tsig_type(id, newdecl, rs) :: rem
     | (Tsig_type(id, decl, rs) :: rem, [s], (Pwith_type _ | Pwith_typesubst _))
@@ -131,10 +141,10 @@ let merge_constraint initial_env loc sg lid constr =
       when Ident.name id = s ->
         (* Check as for a normal with constraint, but discard definition *)
         let newdecl =
-          Typedecl.transl_with_constraint initial_env id None sdecl in
+          Typedecl.transl_with_constraint initial_env id None decl sdecl in
         check_type_decl env id row_id newdecl decl rs rem;
         real_id := Some id;
-        rem
+        make_next_first rs rem
     | (Tsig_module(id, mty, rs) :: rem, [s], Pwith_module lid)
       when Ident.name id = s ->
         let (path, mty') = Typetexp.find_module initial_env loc lid in
@@ -147,7 +157,7 @@ let merge_constraint initial_env loc sg lid constr =
         let newmty = Mtype.strengthen env mty' path in
         ignore(Includemod.modtypes env newmty mty);
         real_id := Some id;
-        rem
+        make_next_first rs rem
     | (Tsig_module(id, mty, rs) :: rem, s :: namelist, _)
       when Ident.name id = s ->
         let newsg = merge env (extract_sig env loc mty) namelist None in
@@ -316,7 +326,8 @@ let check_sig_item type_names module_names modtype_names loc = function
 
 let rec remove_values ids = function
     [] -> []
-  | Tsig_value (id, _) :: rem when List.exists (Ident.equal id) ids -> rem
+  | Tsig_value (id, _) :: rem
+    when List.exists (Ident.equal id) ids -> remove_values ids rem
   | f :: rem -> f :: remove_values ids rem
 
 let rec get_values = function
