@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typetexp.mli 9297 2009-06-12 12:40:55Z garrigue $ *)
+(* $Id$ *)
 
 (* Typechecking of type expressions for the core language *)
 
@@ -43,7 +43,6 @@ type error =
   | Type_arity_mismatch of Longident.t * int * int
   | Bound_type_variable of string
   | Recursive_type
-  | Unbound_class of Longident.t
   | Unbound_row_variable of Longident.t
   | Type_mismatch of (Types.type_expr * Types.type_expr) list
   | Alias_type_mismatch of (Types.type_expr * Types.type_expr) list
@@ -54,8 +53,31 @@ type error =
   | Variant_tags of string * string
   | Invalid_variable_name of string
   | Cannot_quantify of string * Types.type_expr
+  | Multiple_constraints_on_type of string
   | Repeated_method_label of string
+  | Unbound_value of Longident.t
+  | Unbound_constructor of Longident.t
+  | Unbound_label of Longident.t
+  | Unbound_module of Longident.t
+  | Unbound_class of Longident.t
+  | Unbound_modtype of Longident.t
+  | Unbound_cltype of Longident.t
+  | Ill_typed_functor_application of Longident.t
 
 exception Error of Location.t * error
 
 val report_error: formatter -> error -> unit
+
+(* Support for first-class modules. *)
+val transl_modtype_longident: (Location.t -> Env.t -> Longident.t -> Path.t) ref  (* from Typemod *)
+val transl_modtype: (Env.t -> Parsetree.module_type -> Types.module_type) ref (* from Typemod *)
+val create_package_mty: Location.t -> Env.t -> Parsetree.package_type -> (string * Parsetree.core_type) list * Parsetree.module_type
+
+val find_type: Env.t -> Location.t -> Longident.t -> Path.t * Types.type_declaration
+val find_constructor: Env.t -> Location.t -> Longident.t -> Types.constructor_description
+val find_label: Env.t -> Location.t -> Longident.t -> Types.label_description
+val find_value: Env.t -> Location.t -> Longident.t -> Path.t * Types.value_description
+val find_class:  Env.t -> Location.t -> Longident.t -> Path.t * Types.class_declaration
+val find_module: Env.t -> Location.t -> Longident.t -> Path.t * Types.module_type
+val find_modtype: Env.t -> Location.t -> Longident.t -> Path.t * Types.modtype_declaration
+val find_cltype: Env.t -> Location.t -> Longident.t -> Path.t * Types.cltype_declaration
